from Homevee.Exception import InvalidParametersException, DatabaseSaveFailedException
from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database


class Room(Item):
    def __init__(self, name, icon, id=None):
        super(Room, self).__init__()
        self.id = id
        self.name = name
        self.icon = icon

        self.room_data = None

    def get_room_data(self, db: Database = None):
        if db is None:
            db = Database()
        if(self.room_data is None):
            room_data = None #TODO load room data
            self.room_data = room_data
        return self.room_data

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        #TODO implement room deletion
        return False

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            #insert
            if(self.id is None or self.id == ""):
                db.insert("INSERT INTO ROOMS (NAME, ICON) VALUES (:name, :icon)",
                            {'name': self.name, 'icon': self.icon})
            #update
            else:
                db.update("UPDATE ROOMS SET NAME = :name, ICON = :icon WHERE LOCATION = :id",
                            {'name': self.name, 'icon': self.icon, 'id': self.id})
                    # TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save room to database")

    def get_dict(self, fields=None):
        dict = {
            'id': self.id,
            'name': self.name,
            'icon': self.icon
        }

        if(fields is None):
            return dict
        else:
            try:
                output_dict = {}

                for field in fields:
                    output_dict[field] = dict[field]

                return output_dict
            except:
                raise InvalidParametersException("InvalidParams given for Room.get_dict()")

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return Room.load_all_from_db('SELECT * FROM ROOMS WHERE LOCATION IN (%s)' % ','.join('?'*len(ids)),
                                     ids)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = Room(result['NAME'], result['ICON'], result['LOCATION'])
            items.append(item)
        return items

    @staticmethod
    def get_name_by_id(id, db: Database = None):
        if db is None:
            db = Database()
        room = Item.load_from_db(Room, id, db)
        return room.name

    @staticmethod
    def load_all(db: Database):
        if db is None:
            db = Database()

        return Room.load_all_from_db('SELECT * FROM ROOMS', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            name = dict['name']
            icon = dict['icon']
            room = Room(name, icon, id)
            return room
        except:
            raise InvalidParametersException("Room.create_from_dict(): invalid dict")


from Homevee.Item import Item


class Scene(Item):
    def __init__(self):
        super(Scene, self).__init__()

    def run(self):
        """
        Runs the scene
        :return:
        """
        raise NotImplementedError("Scene.run() is not implemented yet")


from Homevee.Exception import InvalidParametersException, AbstractFunctionCallException, ItemNotFoundException
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


class Item():
    def __init__(self):
        pass

    def api_delete(self, db: Database = None) -> dict:
        """
        Deletes the object from the database
        :param db: the database connection
        :return: Status of the request
        """
        try:
            if self.delete(db):
                return Status(type=STATUS_OK).get_dict()
        except:
            pass
        return Status(type=STATUS_ERROR).get_dict()

    def api_save(self, db: Database = None) -> dict:
        """
        Saves the object to the database
        :param db: the database connection
        :return: Status of the request
        """
        try:
            self.save_to_db(db)
            return Status(type=STATUS_OK).get_dict()
        except:
            pass
        return Status(type=STATUS_ERROR).get_dict()

    def delete(self, db: Database = None):
        """
        Deletes the object from the database
        :param db: the database connection
        :return:
        """
        raise AbstractFunctionCallException("Item.delete() is abstract")

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        """
        Saves the object to the database
        :param db: the database connection
        :return:
        """
        raise AbstractFunctionCallException("Item.save_to_db() is abstract")

    def build_dict(self) -> dict:
        """
        Generates a dict from the object
        :return: a dict of the object
        """
        raise AbstractFunctionCallException("Item.build_dict() is abstract")

    def get_dict(self, fields: list = None) -> dict:
        """
        Creates a dict from the object
        :param fields: list of fields to add to the object
        :return: a dict of the object
        """
        dict = self.build_dict()
        if (fields is None):
            return dict
        else:
            try:
                output_dict = {}
                for field in fields:
                    output_dict[field] = dict[field]
                return output_dict
            except:
                raise InvalidParametersException("InvalidParams given for get_dict()")

    @staticmethod
    def load_all_ids_from_db(ids: list, db: Database) -> list:
        """
        Retrieves items identified by the given ids from the database
        :param ids: the ids to retrieves items for
        :param db: the database connection
        :return: the found items
        """
        raise AbstractFunctionCallException("Item.load_all_ids_from_db() is abstract")

    @staticmethod
    def load_all_from_db(query, params, db: Database) -> list:
        """
        Loads all items identified by the given query and the params
        :param query: string of the SQL-query
        :param params: the params for the query
        :param db: the database connection
        :return: a list of found items
        """
        raise AbstractFunctionCallException("Item.load_all_from_db() is abstract")

    @staticmethod
    def load_from_db(module, id: int, db: Database = None):
        """
        Retrieves an item from the database
        :param module: the module to call the method on
        :param id: the id of the item to retrieve
        :param db: the database connection
        :return: the found item
        """
        items = module.load_all_ids_from_db([id], db)
        if ((len(items) == 0) or (str(items[0].id) != str(id))):
            raise ItemNotFoundException("Could not find id: " + id)
        else:
            return items[0]

    @staticmethod
    def create_from_dict(dict: dict):
        """
        Creates an item from the given dict
        :param dict: dict to create an item from
        :return: Item
        """
        raise AbstractFunctionCallException("Item.create_from_dict() is abstract")

    @staticmethod
    def list_to_dict(list: list) -> list:
        """
        Converts a list of items to a list of dicts of items
        :param list: list of items
        :return: list of dicts of items
        """
        data = []
        for item in list:
            data.append(item.get_dict())
        return data


from Homevee.Exception import *
from Homevee.Item.Device.Sensor import Sensor
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import MQTT_SENSOR


class MQTTSensor(Sensor):
    def __init__(self, name, icon, location, save_data, sensor_type, mqtt_device_id, mqtt_value_id, id=None, value=None):
        super(MQTTSensor, self).__init__(name, icon, location, save_data, sensor_type, id=id, value=value)
        self.mqtt_device_id = mqtt_device_id
        self.mqtt_value_id = mqtt_value_id

    def get_device_type(self) -> str:
        return MQTT_SENSOR

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM MQTT_SENSORS WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            #insert
            if(self.id is None or self.id == ""):
                db.insert("INSERT INTO MQTT_SENSORS (ROOM, NAME, ICON, SAVE_DATA,"
                            "TYPE, LAST_VALUE, DEVICE_ID, VALUE_ID) VALUES (:location, :name, :icon, :save_data,"
                            ":sensor_type, :mqtt_device_id, :mqtt_value_id)",
                            {'location': self.location, 'name': self.name, 'icon': self.icon,
                             'save_data': self.save_data, 'sensor_type': self.sensor_type,
                             'mqtt_device_id': self.mqtt_device_id, 'mqtt_value_id': self.mqtt_value_id,})
            #update
            else:
                db.update("UPDATE MQTT_SENSORS SET ROOM = :location, NAME = :name, ICON = :icon,"
                            "SAVE_DATA = :save_data, TYPE = :sensor_type, DEVICE_ID = :mqtt_device_id, "
                                "VALUE_ID = :mqtt_value_id, LAST_VALUE = :value WHERE ID = :id",
                            {'location': self.location, 'name': self.name, 'icon': self.icon,
                             'save_data': self.save_data, 'sensor_type': self.sensor_type,
                             'mqtt_device_id': self.mqtt_device_id, 'mqtt_value_id': self.mqtt_value_id,
                             'value': self.value, 'id': self.id})
                    # TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save room to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'value': self.value,
            'id': self.id,
            'location': self.location,
            'save_data': self.save_data,
            'sensor_type': self.sensor_type,
            'mqtt_device_id': self.mqtt_device_id,
            'mqtt_value_id': self.mqtt_value_id
        }
        return dict

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return MQTTSensor.load_all_from_db('SELECT * FROM MQTT_SENSORS WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = MQTTSensor(result['NAME'], result['ICON'], result['ROOM'], result['SAVE_DATA'],
                                 result['TYPE'], result['DEVICE_ID'], result['VALUE_ID'],
                              result['ID'], result['LAST_VALUE'])
            items.append(item)
        return items

    @staticmethod
    def load_all(db: Database):
        if db is None:
            db = Database()
        return MQTTSensor.load_all_from_db('SELECT * FROM MQTT_SENSORS', {})

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            value = dict['value']
            icon = dict['icon']
            save_data = dict['save_data']
            sensor_type = dict['sensor_type']
            mqtt_device_id = dict['mqtt_device_id']
            mqtt_value_id = dict['mqtt_value_id']

            item = MQTTSensor(name, icon, location, save_data, sensor_type, mqtt_device_id, mqtt_value_id, id, value)

            return item
        except:
            raise InvalidParametersException("Invalid parameters for MQTTSensor.create_from_dict()")


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Cronjob import FixedTimeCronjob
from Homevee.Functions.calendar import get_calendar_day_items
from Homevee.Functions.weather import get_weather
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.NotificationManager import NotificationManager


class OverviewNotificationCronjob(FixedTimeCronjob):
    def __init__(self):
        super(OverviewNotificationCronjob, self).__init__(task_name="OverviewNotificationCronjob")

    def task_to_do(self, *args):
        self.send_overview_notifications()

    def get_seconds_to_wait(self, execution_time=None):
        #wait until midnight + 5 minutes
        t = datetime.datetime.today()

        seconds_to_wait = (24-t.hour)*60*60 - ((t.minute * 60) - t.second) + 5*60 #Warten bis 00:05

        return seconds_to_wait

    def send_overview_notifications(self):
        db = Database()

        users = db.select_all("SELECT * FROM USERDATA", {})

        for user in users:
            day_overview = ""

            weather = get_weather(1)
            weather_text = "Wetter: " + weather[0]['desc'] + ", " + str(weather[0]['temps']['min']) + "°C - " + str(weather[0]['temps']['max']) + "°C"

            today = datetime.datetime.today()
            date_today = today.strftime("%Y-%m-%d")
            calendar = get_calendar_day_items(user['USERNAME'], date_today)
            calendar_items = calendar['calendar_entries']
            if len(calendar_items) == 0:
                calendar_text = "Keine Termine"
            else:
                if len(calendar_items) > 1:
                    calendar_text = str(len(calendar_items)) + " Termine: "
                    for i in range(0, len(calendar_items)):
                        if i == 0:
                            calendar_text += calendar_items[i]['name'] + " um " + calendar_items[i]['start'] + " Uhr"
                        else:
                            calendar_text += ", " + calendar_items[i]['name'] + " um " + calendar_items[i]['start'] + " Uhr"
                else:
                    calendar_text = "Ein Termin: "+calendar_items[0]['name']+" um "+calendar_items[0]['start']+" Uhr"

            day_overview = weather_text+"\n"+calendar_text

            NotificationManager().send_notification_to_users([user['USERNAME']], "Deine Tagesübersicht", day_overview)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Helper.helper_functions import get_my_ip
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def generate_key():
    return "123456"

def save_mqtt_device(user, type, location, id, data, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(location):
        return {'result': 'nopermission'}

    #if type == MQTT_SENSOR:
    #    db.insert("INSERT INTO MQTT_SENSORS")

    item_data = json.loads(data)

    for item in item_data:
        if item['devicetype'] == "sensor":
            db.insert("INSERT INTO MQTT_SENSORS (NAME, ICON, TYPE, ROOM, SAVE_DATA, DEVICE_ID, VALUE_ID, LAST_VALUE) VALUES (:name, :icon, :type, :room, :save_data, :dev_id, :val_id, \"N/A\")",
            {'name': item['name'], 'icon': item['icon'], 'type': item['sensor_type'],
            'room': location, 'save_data': item['save_data'], 'dev_id': id, 'val_id': item['id']})
        
    return Status(type=STATUS_OK).get_dict()

def generate_device_data(user, location, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(location):
        return {'result': 'nopermission'}

    item = db.select_one("SELECT * FROM MQTT_DEVICES ORDER BY ID DESC", {})

    if item is not None:
        new_id = item['ID']+1
    else:
        new_id = 0

    topic = "/home/device/"+str(new_id)

    key = generate_key()

    db.insert("INSERT INTO MQTT_DEVICES (ID, LOCATION, KEY, TOPIC) VALUES (:id, :location, :key, :topic)",
                {'id': new_id, 'location': location, 'key': key, 'topic': topic})

    return {'id': new_id, 'topic': topic, 'key': key, 'ip': get_my_ip(),
            'remoteid': db.get_server_data("REMOTE_ID")}

def add_to_intermediates(id, db: Database = None):
    if db is None:
        db = Database()
        db.insert("INSERT INTO MQTT_DEVICE_INTERMEDIATES (ID) VALUES (:id)", {'id': id})

def is_in_intermediates(id, db: Database = None):
    if db is None:
        db = Database()
        data = db.select_one("SELECT COUNT(*) FROM MQTT_DEVICE_INTERMEDIATES WHERE ID = :id", {'id': id})

        if data['COUNT(*)'] == 0:
            return False
        else:
            return True



try:
    import tensorflow as tf
    TENSORFLOW_INSTALLED = True
    #Logger.log("Tensorflow imported")
except:
    TENSORFLOW_INSTALLED = False
    #Logger.log("Error importing Tensorflow")


class ImageClassifier():
    def __init__(self):
        return

    def load_graph(self, model_file):
        graph = tf.Graph()
        graph_def = tf.GraphDef()

        with open(model_file, "rb") as f:
            graph_def.ParseFromString(f.read())
        with graph.as_default():
            tf.import_graph_def(graph_def)

        return graph

    def read_tensor_from_image_file(self, file_name, input_height=299, input_width=299,
                                    input_mean=0, input_std=255):
        input_name = "file_reader"
        output_name = "normalized"
        file_reader = tf.read_file(file_name, input_name)
        if file_name.endswith(".png"):
            image_reader = tf.image.decode_png(file_reader, channels=3, name='png_reader')
        elif file_name.endswith(".gif"):
            image_reader = tf.squeeze(tf.image.decode_gif(file_reader, name='gif_reader'))
        elif file_name.endswith(".bmp"):
            image_reader = tf.image.decode_bmp(file_reader, name='bmp_reader')
        else:
            image_reader = tf.image.decode_jpeg(file_reader, channels=3, name='jpeg_reader')
        float_caster = tf.cast(image_reader, tf.float32)
        dims_expander = tf.expand_dims(float_caster, 0)
        resized = tf.image.resize_bilinear(dims_expander, [input_height, input_width])
        normalized = tf.divide(tf.subtract(resized, [input_mean]), [input_std])
        sess = tf.Session()
        result = sess.run(normalized)

        return result

    def load_labels(self, label_file):
        label = []
        proto_as_ascii_lines = tf.gfile.GFile(label_file).readlines()
        for l in proto_as_ascii_lines:
            label.append(l.rstrip())
        return label

    def get_graph_sizes(self):
        data = [
            {'text': "0.25 (Sehr schnell)", 'value': '0.25'},
            {'text': "0.50 (Schnell)", 'value': '0.50'},
            {'text': "0.75 (Normal)", 'value': '0.75'},
            {'text': "1.0 (Langsam)", 'value': '1.0'}
        ]

        return data

    def get_image_sizes(self):
        data = [
            {'text': "128 (Sehr schnell)", 'value': '128'},
            {'text': "160 (Schnell)", 'value': '160'},
            {'text': "192 (Normal)", 'value': '192'},
            {'text': "224 (Langsam)", 'value': '224'}
        ]
        return data


#!/usr/bin/python
# -*- coding: utf-8 -*-
import urllib.error
import urllib.parse
import urllib.request

from Homevee.DeviceAPI import philips_hue
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


def rgb_control(user, devicetype, id, mode, brightness, color, db, check_user=True):
    if devicetype == URL_RGB_LIGHT:
        result = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})

        if check_user and not user.has_permission(result['LOCATION']):
            return {'result': 'nopermission'}

        output = urllib.request.urlopen("http://"+result['URL']+color).read()

        return Status(type=STATUS_OK).get_dict()
    elif devicetype == PHILIPS_HUE_LIGHT:
        return philips_hue.set_light_mode(user, id, mode, None, brightness, color, db, check_user=True)

def get_rgb_devices(user, room, db: Database = None):
    if db is None:
        db = Database()
    devices = []

    #URL_RGB_LIGHT
    results = db.select_all("SELECT * FROM URL_RGB_LIGHT WHERE LOCATION = :room", {'room': room})
    for item in results:
        rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': URL_RGB_LIGHT,
        'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
        devices.append(rgb_item)
    #PHILIPS_HUE
    results = db.select_all("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE LOCATION = :room AND TYPE = 'color'", {'room': room})
    for item in results:
        rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': PHILIPS_HUE_LIGHT,
                    'icon': item['ICON'], 'value': {'is_on': item['IS_ON']== 1, 'brightness': int(item['BRIGHTNESS']/255*100), 'color': item['HUE']}}
        devices.append(rgb_item)

    return {'rgb': devices}

def get_rgb_device(user, type, id, db: Database = None):
    if db is None:
        db = Database()

        #URL_RGB_LIGHT
        if type == URL_RGB_LIGHT:
            item = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})
            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': URL_RGB_LIGHT,
            'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
            return rgb_item
        #PHILIPS_HUE
        elif type == PHILIPS_HUE_LIGHT:
            item = db.select_one("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id AND TYPE = 'color'", {'id': id})

            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': PHILIPS_HUE_LIGHT,
                        'icon': item['ICON'], 'value': {'is_on': item['IS_ON']== 1,
                        'brightness': int(item['BRIGHTNESS']/255*100), 'color': item['HUE']}}
            return rgb_item


import os
import traceback

from passlib.handlers.pbkdf2 import pbkdf2_sha512

from Homevee.Exception import ItemNotFoundException, InvalidParametersException, DatabaseSaveFailedException
from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Item.User.Permission import Permission
from Homevee.Utils.Database_NEW import Database


class User(Item):
    def __init__(self, username, hashed_password, ip, at_home, events_last_checked,
                 permissions, dashboard_data, salt, fcm_token, current_location, id=None):
        super(User, self).__init__()

        self.id = id
        self.username = username
        self.hashed_password = hashed_password
        self.ip = ip
        self.events_last_checked = events_last_checked
        self.permissions = permissions
        self.dashboard_data = dashboard_data
        self.salt = salt
        self.fcm_token = fcm_token
        self.current_location = current_location

        self.at_home = True
        if at_home == 1 or at_home is True or at_home == "1":
            self.at_home = True

    def delete(self, db: Database = None):
        try:
            db.delete("DELETE FROM USERDATA WHERE USERNAME == :user", {'user': self.username})
            return True
        except:
            return False

    def has_permission(self, permission):
        for user_permission in self.permissions:
            if user_permission.key == "admin" or user_permission.id == permission:
                return True

        return False

    @staticmethod
    def hash_password(password: str) -> tuple:
        """
        Hashes the given password
        :param password: the password to hash
        :return: a tuple with the hashed password and the used salt
        """
        salt = os.urandom(12).hex()

        hashed_pw = pbkdf2_sha512.encrypt(password + salt, rounds=200000)

        # print "Password: "+password
        # print "Salt: "+salt
        # print "Hashed: "+hashed_pw

        #self.hashed_password = hashed_pw
        #self.salt = salt

        return (hashed_pw, salt)

    def verify(self, password: str) -> bool:
        """
        Checks if this user is verified by the given password
        :param password: the password to verify the user with
        :return: true if verified, false otherwise
        """
        try:
            return pbkdf2_sha512.verify(password + self.salt, self.hashed_password)
        except:
            return False

    def save_to_db(self, db: Database = None):
        try:
            # insert
            if (self.id is None or self.id == ""):
                db.insert("""INSERT INTO USERDATA (USERNAME, PASSWORD, IP, PERMISSIONS, PW_SALT) VALUES 
                            (:username, :password, :ip, :permissions, :salt)""",
                            {'username': self.username, 'password': self.hashed_password, 'ip': self.ip,
                             'permissions': Permission.get_json_list(self.permissions), 'salt': self.salt})
            # update
            else:
                db.update("""UPDATE USERDATA SET PASSWORD = :password, IP = :ip, PERMISSIONS = :permissions,
                            PW_SALT = :salt WHERE ID = :id""",
                            {'password': self.hashed_password, 'ip': self.ip,
                             'permissions': Permission.get_json_list(self.permissions),
                             'salt': self.salt, 'id': self.id})
                #TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save user to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'username': self.username,
            'password': self.hashed_password,
            'ip': self.ip,
            'at_home': self.at_home,
            'events_last_checked': self.events_last_checked,
            'permissions': Permission.list_to_dict(self.permissions),
            'dashboard_data': self.dashboard_data,
            'salt': self.salt,
            'fcm_token': self.fcm_token,
            'current_location': self.current_location
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        items = []
        for result in db.select_all(query, params):
            item = User(result['USERNAME'], result['PASSWORD'], result['IP'], result['AT_HOME'],
                        result['EVENTS_LAST_CHECKED'], Permission.create_list_from_json(result['PERMISSIONS']),
                        result['DASHBOARD_DATA'], result['PW_SALT'], result['FCM_TOKEN'],
                        result['CURRENT_LOCATION'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_by_permission(permission, db: Database = None):
        return User.load_by_permissions([permission], db)

    @staticmethod
    def load_by_permissions(permissions, db: Database = None):
        users = User.load_all(db)
        output = []
        for permission in permissions:
            for user in users:
                if user.has_permission(permission) and user not in users:
                    output.append(user)
        return users

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        return User.load_all_from_db('SELECT * FROM USERDATA WHERE ID IN (%s)' % ','.join('?'*len(ids)),
                                     ids, db)

    @staticmethod
    def load_all_usernames_from_db(usernames, db: Database = None):
        return User.load_all_from_db('SELECT * FROM USERDATA WHERE USERNAME IN (%s)' % ','.join('?'*len(usernames)),
                                     usernames, db)

    @staticmethod
    def load_all(db: Database):
        return User.load_all_from_db('SELECT * FROM USERDATA', {}, db)

    @staticmethod
    #ID is username
    def load_username_from_db(username, db: Database = None):
        users = User.load_all_usernames_from_db([username], db)
        if((len(users) == 0) or (users[0].username != username)):
            raise ItemNotFoundException("Could not find username: " + username)
        else:
            return users[0]

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            username = dict['username']
            password = dict['password']
            ip = dict['ip']
            at_home = dict['at_home']
            events_last_checked = dict['events_last_checked']
            permissions = Permission.list_from_dict(dict['permissions'])
            dashboard_data = dict['dashboard_data']
            salt = dict['salt']
            fcm_token = dict['fcm_token']
            current_location = dict['current_location']
            user = User(username, password, ip, at_home, events_last_checked,
                        permissions, dashboard_data, salt, fcm_token, current_location, id)
            return user
        except:
            raise InvalidParametersException("User.create_from_dict(): invalid dict")


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json

from Homevee.DeviceAPI.heating import get_thermostat_info
from Homevee.Functions.calendar import get_calendar_day_items
from Homevee.Functions.nutrition_data import get_user_nutrition_overview
from Homevee.Functions.sensor_data import get_sensor_data
from Homevee.Functions.weather import get_weather
from Homevee.Helper import Logger
from Homevee.Item.Room import Room
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *

STATIC_DASHBOARD_ITEMS = ['calendar', 'weather', 'tvprogramme', 'drivingtime', 'nutritionmanager']

def get_user_dashboard(user, db: Database = None):
    if db is None:
        db = Database()
    user_dashboard = user.dashboard_data

    if user_dashboard is None:
        data = {'static': [], 'dynamic': []}
    else:
        data = json.loads(user_dashboard)

    dashboard_data = {}
    dashboard_data['static'] = data['static']

    if dashboard_data is not None and dashboard_data != "":

        dashboard_keys = []
        for item in dashboard_data['static']:
            Logger.log(item)
            dashboard_keys.append(item['id'])


        for item in STATIC_DASHBOARD_ITEMS:
            if item not in dashboard_keys:
                Logger.log(item+" not existing")
                dashboard_data['static'].append({'id': item, 'checked': False})

        dashboard_data['dynamic'] = []

        '''
        dynamic_temp_items = data['dynamic']
        devices_map = {}
        dynamic_items = []

        print dynamic_temp_items

        for item in dynamic_temp_items:
            found = False

            if(item['type'] not in devices_map):
                devices_map[item['type']] = []

            if item['id'] not in devices_map[item['type']]:
                devices_map[item['type']].append(item['id'])
            else:
                found = True

            if not found:
                dynamic_items.append(item)

        print dynamic_items
        '''

        devices_map = {}
        for item in data['dynamic']:
            found = False

            if (item['type'] not in devices_map):
                devices_map[item['type']] = []

            if item['id'] not in devices_map[item['type']]:
                devices_map[item['type']].append(item['id'])
            else:
                found = True

            if not found:
                dashboard_data['dynamic'].append(get_device_info(user, item['type'], item['id']))

        return dashboard_data

    return {'result': 'unknownuser'}

def get_user_dashboard_items(user, db: Database = None):
    if db is None:
        db = Database()
    dashboard_array = []

    data = user.dashboard_data

    if data is None:
        return {'dashboard': dashboard_array}

    data = json.loads(data)

    static_items = data['static']

    static_data = {}

    for item in static_items:
        static_data[item['id']] = item['checked']

    dynamic_temp_items = data['dynamic']
    devices_map = {}
    dynamic_items = []

    Logger.log(dynamic_temp_items)

    for item in dynamic_temp_items:
        found = False

        if (item['type'] not in devices_map):
            devices_map[item['type']] = []

        if item['id'] not in devices_map[item['type']]:
            devices_map[item['type']].append(item['id'])
        else:
            found = True

        if not found:
            dynamic_items.append(item)

    Logger.log(dynamic_items)

    # Wetter hinzufügen
    if 'weather' in static_data:
        if static_data['weather']:
            weather_data = get_weather(1)
            dashboard_array.append({'id': 'weather', 'data': weather_data[0]})

    # TV Programm hinzufügen
    if 'tvprogramme' in static_data:
        if static_data['tvprogramme']:
            dashboard_array.append({'id': 'tvprogramme', 'data': None})

    # Kalender hinzufügen
    if 'calendar' in static_data:
        if static_data['calendar']:
            calendar_entries = get_calendar_day_items(user, datetime.datetime.now().strftime("%Y-%m-%d"))

            items = calendar_entries['calendar_entries']

            data = None
            for item in items:
                start = datetime.datetime.strptime(item['start'], '%H:%M')
                dnow = datetime.datetime.now()

                if (dnow.time() > start.time()):
                    continue

                if data is None:
                    data = item['name'] + " um " + item['start']
                else:
                    data += '\n' + item['name'] + " um " + item['start'] + " Uhr"

            if data is None:
                data = 'Keine Termine heute'

            dashboard_array.append({'id': 'calendar', 'data': data})

    # Fahrzeit
    if 'drivingtime' in static_data:
        if static_data['drivingtime']:
            dashboard_array.append({'id': 'drivingtime', 'data': '27 min'})

    # Ernährungsmanager
    try:
        if 'nutritionmanager' in static_data:
            if static_data['nutritionmanager']:
                data_item = get_user_nutrition_overview(user)

                if 'nutrition_day_data' in data_item['data']:
                    calories_left = data_item['data']['nutrition_day_data']['calories_left']
                    if (calories_left >= 0):
                        text = str(calories_left) + ' Kalorien übrig'
                    else:
                        text = str(calories_left) + ' Kalorien über deinem Ziel'

                    dashboard_array.append({'id': 'nutritionmanager', 'data': text})
                else:
                    dashboard_array.append({'id': 'nutritionmanager',
                                            'data': "Ernährungsmanager noch nicht eingerichtet"})
    except:
        pass

    if len(dynamic_items) > 0:
        dashboard_array.append({'id': 'favouritedevices', 'data': None})

    return {'dashboard': dashboard_array}

def edit_user_dashboard(username, dashboard_data, db: Database = None):
    if db is None:
        db = Database()


    dashboard_data = json.loads(dashboard_data)

    #Check duplicates

    dynamic_temp_items = dashboard_data['dynamic']
    devices_map = {}
    dynamic_items = []

    Logger.log(dynamic_temp_items)

    for item in dynamic_temp_items:
        found = False

        if (item['type'] not in devices_map):
            devices_map[item['type']] = []

        if item['id'] not in devices_map[item['type']]:
            devices_map[item['type']].append(item['id'])
        else:
            found = True

        if not found:
            dynamic_items.append(item)

    Logger.log(dynamic_items)

    dashboard_data['dynamic'] = dynamic_items

    #Check duplicates end

    dashboard_data_json = json.dumps(dashboard_data)

    db.update("UPDATE USERDATA SET DASHBOARD_DATA = :data WHERE USERNAME == :username",
                {'data': dashboard_data_json, 'username': username})

    return get_user_dashboard(username)

def get_user_favourite_devices(user, db: Database = None):
    if db is None:
        db = Database()
    data = json.loads(user.dashboard_data)

    dynamic_items = data['dynamic']

    Logger.log("data: "+str(data))

    fav_array = get_dynamic_items(user, dynamic_items)

    return fav_array

def get_dynamic_items(user, dynamic_items, db: Database = None):
    if db is None:
        db = Database()
    rooms = Room.load_all(db)

    favourites = {}

    room_map = {}

    for room in rooms:
        if(user.has_permission(room.id)):
            favourites[int(room.id)] = []
            room_map[room.id] = room.get_dict()
        else:
            continue

    types = {}

    for item in dynamic_items:
        #print item
        if item['type'] not in types:
            types[item['type']] = []

        types[item['type']].append(item['id'])
        Logger.log(types)

    for type in types:
        ids = types[type]
        if len(ids) > 0:
            for id in ids:
                item = get_device_info(user, type, id)

                if item is None:
                    continue

                Logger.log(favourites)

                favourites[int(item['room'])].append(item)

    output = []
    for location in list(favourites.keys()):
        room_item = room_map[location]

        if len(favourites[location]) > 0:
            room_item['devices'] = favourites[location]
            output.append(room_item)

    return output

def get_device_info(user, type, id, db: Database = None):
    if db is None:
        db = Database()
    Logger.log(type + " - " + id)

    device_data = None

    try:
        if type == FUNKSTECKDOSE:
            item = db.select_one("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE == :id", {'id': id})
            device_data = {'room': item['ROOM'], 'name': item['NAME'], 'icon': item['ICON'],
                           'value': item['ZUSTAND'] == "1",
                           'type': 'switch', 'devicetype': type, 'id': id}
        elif type == PHILIPS_HUE_LIGHT:
            item = db.select_one("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID == :id", {'id': id})
            device_data = {'room': item['LOCATION'], 'name': item['NAME'],
                           'icon': item['ICON'],
                           'value': {'is_on': False, 'brightness': 100, 'color': item['HUE']},
                           'type': 'rgb', 'devicetype': type, 'id': id}
        #URL-Geräte
        elif type == URL_RGB_LIGHT:
            item = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})
            device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'], 'type': 'rgb',
                           'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
        elif type == URL_TOGGLE:
            item = db.select_one("SELECT * FROM URL_TOGGLE WHERE ID = :id",{'id': id})
            device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'],
                                'icon': item['ICON'], 'value': '', 'type': 'toggle'}
        #Z-Wave
        elif type == ZWAVE_THERMOSTAT:
            item = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID == :id", {'id': id})
            data = get_thermostat_info(user, item['RAUM'], type, item['THERMOSTAT_ID'])
            device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                    'icon': item['ICON'], 'room': item['RAUM'], 'type': 'heating',
                    'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
        elif type == ZWAVE_DIMMER:
            item = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID == :id", {'id': id})
            device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                    'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                    'value': item['VALUE']}
        elif type == ZWAVE_POWER_METER:
            item = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE DEVICE_ID == :id", {'id': id})
            device_data = {'name': item['NAME'], 'id': item['DEVICE_ID'],
                    'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                    'value': item['VALUE']}
        elif type == ZWAVE_SENSOR:
            item = db.select_one("SELECT * FROM ZWAVE_SENSOREN WHERE ID == :id", {'id': id})
            device_data = {'name': item['SHORTFORM'], 'id': item['ID'],
                    'icon': item['ICON'], 'room': item['RAUM'], 'type': 'value',
                           'value': get_sensor_data(user, item['RAUM'], type, id, 1)}
        elif type == ZWAVE_SWITCH:
            item = db.select_one("SELECT * FROM ZWAVE_SWITCHES WHERE THERMOSTAT_ID == :id", {'id': id})
            device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                    'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                    'value': item['VALUE']}
        #MAX Cube
        elif type == MAX_THERMOSTAT:
            item = db.select_one("SELECT * FROM MAX_THERMOSTATS WHERE ID == :id", {'id': id})
            data = get_thermostat_info(user, item['RAUM'], type, id)
            device_data = {'name': item['NAME'], 'id': item['ID'], 'room': item['RAUM'], 'type': 'heating',
                    'icon': item['ICON'],
                    'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
        #Wake on Lan
        elif type == WAKE_ON_LAN:
            item = db.select_one("SELECT * FROM WAKE_ON_LAN WHERE DEVICE == :id", {'id': id})
            device_data = {'name': item['NAME'], 'id': item['DEVICE'], 'icon': item['ICON'], 'room': item['LOCATION'],
                           'value': '', 'type': 'wakeonlan'}
        elif type == XBOX_ONE_WOL:
            item = db.select_one("SELECT * FROM XBOX_ONE_WOL WHERE ID == :id", {'id': id})
            device_data = {'name': item['NAME'], 'id': item['ID'], 'icon': item['ICON'], 'room': item['LOCATION'],
                           'value': '', 'type': 'wakeonlan'}

        #MQTT Sensor
        elif type == MQTT_SENSOR:
            item = db.select_one("SELECT * FROM MQTT_SENSORS WHERE ID == :id", {'id': id})
            device_data = {'name': item['NAME'], 'id': item['ID'],
                           'icon': item['ICON'], 'room': item['ROOM'], 'type': 'value',
                           'value': get_sensor_data(user, item['ROOM'], type, id, 1)}
    except:
        Logger.log("Error loading device")

    if device_data is None or not user.has_permission(device_data['room']):
        return None

    device_data['roomname'] = Room.get_name_by_id(device_data['room'])
    device_data['devicetype'] = type

    return device_data

    '''//DIY
		case "DIY Gerät":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_DEVICES' WHERE DEVICE == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['ROOM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => "", "type" => "button");
			break;
		case "DIY Sensor":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_SENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getSensorData($query['RAUM'], $type, $id, "1", $db), "type" => "value");
		case "DIY Fensterkontakt":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_REEDSENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "reed");
			break;
		case "DIY Anwesenheitssensor":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_PRESENCESENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "presence");
			break;
		case "DIY Schalter":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_SWITCHES' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getModes($query['RAUM'], $type, $id, $db) === "1", "type" => "switch");
			break;
		case "IP Kamera":
			$dbQuery = $db->prepare("SELECT * FROM 'IP_CAMERA' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			
			$ip = $query['IP'];
			$port = $query['PORT'];
				
			if($port != null && $port != ""){
				$port = ":$port";
			}
				
			$path = $query['PATH'];
				
			$url = "http://$ip$port$path";
			
			$devicedata = array('room' => $query['LOCATION'], 'name' => $query['NAME'], 'icon' => $query['ICON'],
			'value' => "", 'url' => $url, 'username' => $query['USERNAME'], 'password' => $query['PASSWORD'],
			'type' => "ipcamera");
			break;
			
		default:
			return null;
	}
	
	if(!hasPermission($devicedata['room'], $db) || $devicedata['room'] == null){
		return null;
	}
	else{
		$devicedata['roomname'] = $roomArray[$devicedata['room']];
		$devicedata['devicetype'] = $type;
		$devicedata['id'] = $id;
		
		return $devicedata;
	}
}
?>
    '''



#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json

import requests
from dateutil.relativedelta import relativedelta

from Homevee.Helper import Logger
from Homevee.Helper.helper_functions import load_remote_data
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def get_nutrition_data(user, keyword, db: Database = None):
    if db is None:
        db = Database()
    data = {'action': 'findsingleitem', 'term': keyword}

    Logger.log("Nutrition item search: "+keyword)

    r = requests.post("https://food.Homevee.de/api.php", data=data)
    # Logger.log(r.status_code, r.reason)

    if (r.status_code != 200):
        return None

    response = r.text

    #print response

    data = json.loads(response)

    if(data is None):
        return None

    if('food' not in data):
        return None

    nutrition_data = data['food']

    if(nutrition_data is None):
        return None

    if(len(nutrition_data) is 0):
        return False

    return nutrition_data[0]

def submit_food(user, name, calories, portionsize, portionunit, protein, fat, saturated, unsaturated, carbs, sugar, ean, db: Database = None):
    if db is None:
        db = Database()
    remote_data = load_remote_data(user)
    remote_id = remote_data['remote_id']

    if(remote_id is None or remote_id == ""):
        return {'result': 'noremoteid'}

    data = {'action': 'submitfood', 'name': name, 'calories': calories, 'portionsize': portionsize, 'portionunit': portionunit,
            'protein': protein, 'fat': fat, 'saturated': saturated, 'unsaturated': unsaturated, 'carbs': carbs,
            'sugar': sugar, 'ean': ean, 'user_remote_id': remote_id}

    r = requests.post("https://food.Homevee.de/api.php", data=data)
    #Logger.log(r.status_code, r.reason)

    if(r.status_code != 200):
        return {"result": "error"}

    result = json.loads(r.text)

    if("result" in result):
        if(result['result'] == "ok"):
            return Status(type=STATUS_OK).get_dict()

    return Status(type=STATUS_ERROR).get_dict()

def get_user_day_nutrition_data(user, db: Database = None):
    if db is None:
        db = Database()
    return

def get_user_nutrition_overview(user, db: Database = None):
    if db is None:
        db = Database()
    user_profile = get_user_fitness_profile(user)

    Logger.log(user_profile)

    output = {}

    if (user_profile is False):
        has_profile = False
    else:
        has_profile = True

        results = db.select_all("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                    {'user': user.username, 'date': datetime.datetime.today().strftime("%d.%m.%Y")})

        eaten_calories_today = 0
        eaten_fat_today = 0
        eaten_carbs_today = 0
        eaten_protein_today = 0

        for item in results:
            amount = (float(item['EATEN_PORTION_SIZE'])/float(item['PORTIONSIZE']))

            eaten_calories_today += item['CALORIES']*amount
            eaten_fat_today += item['FAT']*amount
            eaten_carbs_today += item['CARBS']*amount
            eaten_protein_today += item['PROTEIN']*amount

        #print datetime.datetime.today().strftime("%Y-%m-%d")

        nutrition_distribution = {'fat': eaten_fat_today, 'carbs': eaten_carbs_today,
                                  'protein': eaten_protein_today}
        nutrition_day_data = {'calories_goal': int(user_profile['caloriesgoal']), 'calories_left': int(user_profile['caloriesgoal']-eaten_calories_today), 'nutrition_distribution': nutrition_distribution}

        output['nutrition_day_data'] = nutrition_day_data

    weight_progress = {}

    output['has_profile'] = has_profile
    output['weight_progress'] = weight_progress

    return {'data': output}

def save_user_settings(user, height, weight, birthdate, mode, activity, db: Database = None):
    if db is None:
        db = Database()
    calories_goal = get_calorie_need(birthdate, weight, height, activity, mode)

    protein_goal, carbs_goal, fat_goal = get_macro_distribution(calories_goal)

    water_goal = 2.0

    add_user_weight(user, weight)
    result = db.select_one("SELECT COUNT(*) FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': user.username})

    result = result['COUNT(*)']

    param_array = {'cals': calories_goal, 'user': user.username, 'fat': fat_goal, 'saturated': 0, 'carbs': carbs_goal,
                   'sugar': 0, 'protein': protein_goal, 'water': water_goal, 'birthdate': birthdate, 'height': height,
                   'weight': weight, 'activity':activity, 'diet': mode}

    if(result == 0):
        db.insert("INSERT INTO USER_FITNESS_DATA (user, CALORIE_GOAL, FAT_GOAL, SATURATED_GOAL, CARBS_GOAL, SUGAR_GOAL, PROTEIN_GOAL, WATER_GOAL, BIRTHDATE, HEIGHT, ACTIVITY_MODE, DIET_MODE) VALUES (:user, :cals, :fat, :saturated, :carbs, :sugar, :protein, :water, :birthdate, :height, :activity, :diet)", param_array)
    else:
        db.update("UPDATE USER_FITNESS_DATA SET CALORIE_GOAL = :cals, FAT_GOAL = :fat, SATURATED_GOAL = :saturated, CARBS_GOAL = :carbs, SUGAR_GOAL = :sugar, PROTEIN_GOAL = :protein, WATER_GOAL = :water, BIRTHDATE = :birthdate, HEIGHT = :height, ACTIVITY_MODE = :activity, DIET_MODE = :diet WHERE USERNAME = :user", param_array)

    return {'result': 'ok', 'calories_goal': calories_goal, 'protein_goal': protein_goal, 'carbs_goal': carbs_goal, 'fat_goal': fat_goal}

def get_macro_distribution(calories):

    protein_factor = 0.25
    carbs_factor = 0.5
    fat_factor = 0.25

    protein_goal = (calories/4)*protein_factor
    carbs_goal = (calories/4)*carbs_factor
    fat_goal = (calories/8)*fat_factor

    return int(protein_goal), int(carbs_goal), int(fat_goal)

def load_user_settings(user, db: Database = None):
    if db is None:
        db = Database()
        data = {}

        #{'sitting': 1.2, 'lightactivity': 1.35, 'moderateactivity': 1.55, 'highactivity': 1.75, 'extremeactivity': 1.95}

        data['activities'] = [
            {'title': 'Sitzend', 'description': 'wenig/kein Training + Bürotätigkeit', 'key': 'sitting'},
            {'title': 'leichte Aktivität', 'description': 'geringe tägliche Aktivität + leichtes Training 1-3 Mal/ Woche', 'key': 'lightactivity'},
            {'title': 'moderate Aktivität', 'description': 'moderate tägliche Aktivität + moderates Training 3-5 Mal/ Woche', 'key': 'moderateactivity'},
            {'title': 'hohe Aktivität', 'description': 'körperlich anspruchsvoller Lifestyle + schweres Training/Sport 6-7 Mal/ Woche', 'key': 'highactivity'},
            {'title': 'extrem hohe Aktivität', 'description': 'hartes tägliches Training/Sport und/oder körperlich beanspruchender Job', 'key': 'extremeactivity'}
        ]

        data['modes'] = [
            {'title': 'stark Abnehmen', 'key': 'harddiet'},
            {'title': 'Abnehmen', 'key': 'diet'},
            {'title': 'Gewicht halten', 'key': 'hold'},
            {'title': 'Muskeln aufbauen', 'key': 'gainmuscles'}
        ]

        result = db.select_one("SELECT * FROM USER_WEIGHT_DATA WHERE USERNAME = :user ORDER BY DATE DESC", {'user': user.username})
        weight_data = result['WEIGHT']

        result = db.select_one("SELECT * FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': user.username})

        activity_title = None
        for item in data['activities']:
            if(item['key'] == result['ACTIVITY_MODE']):
                activity_title = item['title']
                break

        mode_title = None
        for item in data['modes']:
            if(item['key'] == result['DIET_MODE']):
                mode_title = item['title']
                break

        settings = {'weight': weight_data, 'height': result['HEIGHT'], 'birthdate': result['BIRTHDATE'],'activity': result['ACTIVITY_MODE'],
                    'activity_title': activity_title, 'mode': result['DIET_MODE'], 'mode_title': mode_title}

        data['settings'] = settings

        return {'data': data}

def add_user_weight(user, weight, db: Database = None):
    if db is None:
        db = Database()
    date = datetime.datetime.today().__format__("%Y-%m-%d")

    db.insert("INSERT INTO USER_WEIGHT_DATA (user, DATE, WEIGHT) VALUES (:user, :date, :weight)",
               {'user': user.username, 'date': date, 'weight': weight})

def delete_food_item(user, id, db: Database = None):
    if db is None:
        db = Database()
    db.delete("DELETE FROM NUTRITION_DATA WHERE ID = :id AND USER = :user",
                {'id': id, 'user': user.username})

    return Status(type=STATUS_OK).get_dict()

def get_calorie_need(birthdate, weight, height, movement_mode, diet_mode):
    # (9.99 x Körpergewicht (in kg)) + (6.25 x Körpergröße (in cm)) – (4.92 x Alter (in Jahren)) – 161

    birthdate = datetime.datetime.strptime(birthdate, "%Y-%m-%d")
    today = datetime.datetime.today()
    age = relativedelta(today, birthdate).years

    base_calories = (9.99 * int(weight)) + (6.25 * int(height)) - (4.92 * age) - 161

    movement_factor = 1.2
    movement_map = {'sitting': 1.2, 'lightactivity': 1.35, 'moderateactivity': 1.55, 'highactivity': 1.75, 'extremeactivity': 1.95}
    if(movement_mode in movement_map):
        movement_factor = movement_map[movement_mode]
    calories_with_movement = base_calories * movement_factor

    diet_factor = 1
    diet_map = {'harddiet': 0.8, 'diet': 0.9, 'hold': 1.0, 'musclegain': 1.1}
    if(diet_mode in diet_map):
        diet_factor = diet_map[diet_mode]
    Logger.log(diet_mode)
    diet_calories = calories_with_movement * diet_factor

    calories_needed = diet_calories

    return int(calories_needed)

def get_user_fitness_profile(user, db: Database = None):
    if db is None:
        db = Database()

    result = db.select_one("SELECT * FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': user.username})

    if(result is None):
        return False

    data = {'caloriesgoal': result['CALORIE_GOAL'], 'fatgoal': result['FAT_GOAL'], 'saturatedgoal': result['SATURATED_GOAL'],
            'carbsgoal': result['CARBS_GOAL'], 'sugargoal': result['SUGAR_GOAL'], 'proteingoal': result['PROTEIN_GOAL'],
            'watergoal': result['WATER_GOAL']}

    return data

def get_user_weight_progress(user, start, end, db: Database = None):
    if db is None:
        db = Database()
    start = datetime.datetime.strptime(start, "%d.%m.%Y").strftime("%Y-%m-%d")
    end = datetime.datetime.strptime(end, "%d.%m.%Y").strftime("%Y-%m-%d")

    weight_items = []

    results = db.select_all("SELECT WEIGHT, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM USER_WEIGHT_DATA WHERE USERNAME = :user AND DATE >= :start AND DATE <= :ende ORDER BY DATE ASC",
                {'user': user.username, 'start': start, 'end': end})

    for item in results:
        weight_items.append({'date': item['DATETIME'], 'weight': item['WEIGHT']})

    return {'progress': weight_items}

def get_user_day_nutrition_items(user, date, db: Database = None):
    if db is None:
        db = Database()
    daytime_items = {'morning': [], "noon": [], "evening": [], "snacks": []}

    results = db.select_all("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                {'user': user.username, 'date': date})

    for item in results:
        food_item = {'id': item['ID'], 'daytime': item['DAYTIME'], 'name': item['FOOD_NAME'], 'eatenportionsize': item['EATEN_PORTION_SIZE'],
                     'portionsize': item['PORTIONSIZE'], 'portionunit': item['PORTIONUNIT'], 'calories': item['CALORIES'],
                     'fat': item['FAT'], 'saturated': item['SATURATED'], 'unsaturated': item['UNSATURATED'],
                     'carbs': item['CARBS'], 'sugar': item['SUGAR'], 'protein': item['PROTEIN'], 'date': item['DATE']}

        if(item['DAYTIME'] in daytime_items):
            daytime_items[item['DAYTIME']].append(food_item)
        else:
            daytime_items['snacks'].append(food_item)

    calories_goal = get_user_fitness_profile(user)['caloriesgoal']

    return {'items': daytime_items, 'caloriesgoal': calories_goal}

def move_item(user, id, date, daytime, deleteold, db: Database = None):
    if db is None:
        db = Database()
    item = db.select_all("SELECT * FROM NUTRITION_DATA WHERE ID = :id", {'id': id})

    if(item is None):
        return {'result': 'itemnotexisting'}

    name = item['FOOD_NAME']
    eatenportionsize = item['EATEN_PORTION_SIZE']
    portionsize = item['PORTIONSIZE']
    portionunit = item['PORTIONUNIT']
    calories = item['CALORIES']
    fat = item['FAT']
    saturated = item['SATURATED']
    unsaturated = item['UNSATURATED']
    carbs = item['CARBS']
    sugar = item['SUGAR']
    protein = item['PROTEIN']

    add_edit_user_day_nutrition_item(user, None, date, daytime, name, eatenportionsize, portionsize, portionunit,
                                     calories, fat, saturated, unsaturated, carbs, sugar, protein)

    if deleteold == "true" or deleteold == True or deleteold == 1 or deleteold == "1":
        db.delete("DELETE FROM NUTRITION_DATA WHERE ID = :id", {'id': id})

    return Status(type=STATUS_OK).get_dict()

def add_edit_user_day_nutrition_item(user, id, date, daytime, name, eatenportionsize, portionsize, portionunit, calories, fat, saturated, unsaturated, carbs, sugar, protein, db: Database = None):
    if db is None:
        db = Database()
    add_new = (id == None or id == "" or id == "-1")

    param_array = {'user': user.username, 'date': date, 'daytime': daytime, 'name': name, 'eatenportionsize': eatenportionsize,
                   'portionsize': portionsize, 'portionunit': portionunit, 'calories': calories, 'fat': fat,
                   'saturated': saturated, 'unsaturated': unsaturated, 'carbs': carbs, 'sugar': sugar, 'protein': protein}

    if add_new:
        db.insert("INSERT INTO NUTRITION_DATA (USER, DATE, DAYTIME, FOOD_NAME, EATEN_PORTION_SIZE, PORTIONSIZE, PORTIONUNIT, CALORIES, FAT, SATURATED, UNSATURATED, CARBS, SUGAR, PROTEIN) VALUES (:user, :date, :daytime, :name, :eatenportionsize, :portionsize, :portionunit, :calories, :fat, :saturated, :unsaturated, :carbs, :sugar, :protein)",
                    param_array)
    else:
        param_array['id'] = id

        db.update("UPDATE NUTRITION_DATA SET USER = :user, DATE = :date, DAYTIME = :daytime, FOOD_NAME = :name, EATEN_PORTION_SIZE = :eatenportionsize, PORTIONSIZE = :portionsize, PORTIONUNIT = :portionunit, CALORIES = :calories, FAT = :fat, SATURATED = :saturated, UNSATURATED = :unsaturated, CARBS = :carbs, SUGAR = :sugar, PROTEIN = :protein WHERE ID = :id",
                    param_array)

    return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

import requests

from Homevee.Item import Item
from Homevee.Item.Gateway import Gateway, Z_WAVE_GATEWAY
from Homevee.Utils.Database_NEW import Database


def do_zwave_request(path, db: Database = None):
    if db is None:
        db = Database()
    try:
        gateway = Item.load_from_db(Gateway, Z_WAVE_GATEWAY, db)
        port = ""
        if (gateway.port != None and gateway.port != ""):
            port = ":" + str(gateway.port)

        url = "http://" + gateway.key1 + ':' + gateway.key2 + '@' + gateway.ip + port + path

        #print url

        headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
        request = requests.get(url, headers=headers)
        data = request.content

        #print data

        json_data = json.loads(data)

        #print url, json_data

        return json_data
    except:
        return None


import os

from Homevee.DeviceAPI import xbox_wol
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def wake_on_lan(user, id, db, check_user=True):
    device = db.select_all("SELECT * FROM WAKE_ON_LAN WHERE DEVICE == :id", {'id': id})
    if check_user and not user.has_permission(device['LOCATION']):
        return {'result': 'nopermission'}

    os.system("sudo wakeonlan "+device['MAC_ADDRESS'])

    return Status(type=STATUS_OK).get_dict()

def get_wol_devices(user, room, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    devices = []

    results = db.select_all("SELECT * FROM WAKE_ON_LAN WHERE LOCATION == :room", {'room': room})

    for data in results:
        devices.append({'name': data['NAME'], 'id': data['DEVICE'], 'icon': data['ICON']})

    return {'devices': devices}

def wake_xbox_on_lan(user, id, db, check_user=True):
        device = db.select_all("SELECT * FROM XBOX_ONE_WOL WHERE ID == :id",
                    {'id': id})

        if check_user and not user.has_permission(device['LOCATION']):
            return {'result': 'nopermission'}

        ip = device['IP_ADDRESS']
        live_id = device['XBOX_LIVE_ID']

        xbox_wol.xbox_wake_up(ip, live_id)

        return Status(type=STATUS_OK).get_dict()

def get_xbox_devices(user, room, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    devices = []
    results = db.select_all("SELECT * FROM XBOX_ONE_WOL WHERE LOCATION == :room", {'room': room})

    for data in results:
        devices.append({'name': data['NAME'], 'id': data['ID'], 'icon': data['ICON']})

    return {'devices': devices}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Item.Device import Device
from Homevee.Item.HeatingSchemeItem import HeatingSchemeItem
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def add_edit_heating_scheme_item(user, id, time, value, active, days, data, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    with db:
        cur = db.cursor()

        params = {'time': time, 'value': value, 'active': active}

        if id == "" or id is None:
            db.insert("INSERT INTO HEATING_SCHEME (TIME, VALUE, ACTIVE) VALUES (:time, :value, :active)",
                        params)

            id = cur.lastrowid
        else:
            params['id'] = id
            db.update("UPDATE HEATING_SCHEME SET TIME = :time, VALUE = :value, ACTIVE = :active WHERE ID = :id",
                params)

        #HEATING_SCHEME_DAYS bearbeiten
        db.delete("DELETE FROM 'HEATING_SCHEME_DAYS' WHERE HEATING_SCHEME_ID = :id", {'id': id})

        day_array = json.loads(days)

        for day in day_array:
            db.insert("INSERT INTO 'HEATING_SCHEME_DAYS' (HEATING_SCHEME_ID, WEEKDAY_ID) VALUES (:id, :weekday_id)",
                {'id': id, 'weekday_id': day})

        #HEATING_SCHEME_DEVICES bearbeiten
        db.delete("DELETE FROM 'HEATING_SCHEME_DEVICES' WHERE ID = :id", {'id': id})

        device_array = json.loads(data)

        for device in device_array['devices']:
            db.insert("INSERT INTO 'HEATING_SCHEME_DEVICES' (ID, LOCATION, TYPE, DEVICE_ID) VALUES (:id, :location, :type, :device_id)",
                {'id': id, 'location': device['location'], 'type': device['type'], 'device_id': device['id']})

        return Status(type=STATUS_OK).get_dict()

def delete_heating_scheme_item(user, id, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    heating_scheme_item = Device.load_from_db(HeatingSchemeItem, id)
    return heating_scheme_item.api_delete()

def get_heating_scheme_items(user, day, rooms, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return "noadmin"

    params = {'day': day}
    query_in_rooms_string = ""

    if rooms is not None and rooms != "":
        params['rooms'] = rooms
        query_in_rooms_string = "AND LOCATION IN (:rooms)"

    results = db.select_all("SELECT HEATING_SCHEME.ID, TIME, VALUE, ACTIVE, WEEKDAY_ID, ROOMS.NAME as LOCATION, TYPE, DEVICE_ID FROM HEATING_SCHEME, HEATING_SCHEME_DAYS, HEATING_SCHEME_DEVICES, ROOMS WHERE HEATING_SCHEME.ID = HEATING_SCHEME_DAYS.HEATING_SCHEME_ID AND HEATING_SCHEME.ID = HEATING_SCHEME_DEVICES.ID AND HEATING_SCHEME_DEVICES.LOCATION = ROOMS.LOCATION AND WEEKDAY_ID = :day "+query_in_rooms_string+" ORDER BY TIME",
                params)

    heating_scheme_items = {}
    heating_scheme_data = {}

    for result in results:
        if result['ID'] not in heating_scheme_data:
            heating_scheme_data[result['ID']] = []

        heating_scheme_data[result['ID']].append({'location': result['LOCATION'], 'type': result['TYPE'], 'device':result['DEVICE_ID']})

        heating_scheme_items[result['ID']] = {'time': result['TIME'], 'value': result['VALUE'], 'isactive': (True if result['ACTIVE']=="true" else False)}

    for result in results:
        heating_scheme_items[result['ID']]['data'] = heating_scheme_data[result['ID']]

    return {'heatingscheme': heating_scheme_items}

def get_heating_scheme_item_data(username, id, db: Database = None):
    if db is None:
        db = Database()
    heating_scheme_item = {}

    params = {'id': id}

    #Tage abfragen
    chosen_days = []
    results = db.select_all("SELECT WEEKDAY_ID FROM HEATING_SCHEME_DAYS WHERE HEATING_SCHEME_ID == :id", params)
    for day in results:
        chosen_days.append(int(day['WEEKDAY_ID']))
    heating_scheme_item['days'] = chosen_days

    #Geräte abfragen
    devices = []
    results = db.select_all("SELECT * FROM HEATING_SCHEME_DEVICES WHERE ID == :id", params)
    for device in results:
        devices.append({'id': device['DEVICE_ID'], 'type': device['TYPE'], 'location': device['LOCATION']})
    heating_scheme_item['devicearray'] = json.dumps({'devices': devices})

    #Daten
    scheme_item_data = db.select_one("SELECT * FROM HEATING_SCHEME WHERE ID == :id", params)

    heating_scheme_item['value'] = float(scheme_item_data['VALUE'])
    heating_scheme_item['time'] = scheme_item_data['TIME']
    heating_scheme_item['active'] = scheme_item_data['ACTIVE']

    return {'heatingschemeitem': heating_scheme_item}

def set_heating_scheme_item_active(user, id, active, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()
    try:
        db.update("UPDATE HEATING_SCHEME SET ACTIVE = :active WHERE ID = :id", {'active': active, 'id': id})
        return Status(type=STATUS_OK).get_dict()
    except:
        pass
    return Status(type=STATUS_ERROR).get_dict()

def is_heating_scheme_active(user, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    is_active = db.get_server_data("HEATING_SCHEME_ACTIVE")

    return {'isactive': is_active=="true"}

def set_heating_scheme_active(user, active, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    try:
        db.set_server_data("HEATING_SCHEME_ACTIVE", active)
        return Status(type=STATUS_OK).get_dict()
    except:
        pass
    return Status(type=STATUS_ERROR).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import codecs
import urllib.error
import urllib.error
import urllib.parse
import urllib.parse
import urllib.request
import urllib.request

from Homevee.Helper import translations, Logger
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Helper import generate_string
from Homevee.VoiceAssistant.Modules import VoiceModule

GREETINGS = ['hallo', 'hey', 'na', 'hi']

THANKS = ['danke', 'dank', 'dankeschön']

class VoiceConversationModule(VoiceModule):
    def get_context_key(self):
        return "VOICE_CONVERSATION"

    def get_priority(self):
        return 0

    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return []

    def get_label(self):
        return "conversation"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        Logger.log("conversation: "+text)
        return self.conversation(username, text, context, db)

    def conversation(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        url = Helper.SMART_API_PATH + "/?action=plaintext&language=" + translations.LANGUAGE + "&text=" + urllib.parse.quote(codecs.encode(text, 'utf-8'))
        Logger.log("calling: "+url)
        output = urllib.request.urlopen(url).read()

        #print(url)

        #print(output)

        if output is None or output == '' or len(output)==0:
            data = [
                [[['Es ', 'tut '], 'Tut '], 'mir ', ['echt ', 'sehr ', ''], ['Leid. ', 'Leid, '+username+'. '],
                ['Da ', 'Dabei ', 'Damit '], 'kann ', 'ich ', 'dir ', ['leider ', ''], ['noch ', ''], 'nicht ', ['helfen', 'weiterhelfen'], '.']
            ]
            output = generate_string(data)
        else:
            output = output.decode("utf-8")

        result = {'msg_speech': output, 'msg_text': output}

        return result

    def contains(self, text, array):
        words = text.split(' ')

        for item in array:
            if item in words:
                return True

        return False


# https://github.com/olucurious/PyFCM

from Homevee.Helper.HomeveeCloud import HomeveeCloudWrapper
from Homevee.Item.User import User
from Homevee.Utils.Database_NEW import Database


class NotificationManager():
    def __init__(self):
        return

    def send_notification(self, registration_ids: list, message_title: str,
                          message_body: str, click_action: dict = None):
        """
        Sends a notification to the ids
        :param registration_ids: the recipients ids
        :param message_title: the title of the notification
        :param message_body: the body of the notification
        :param click_action: the on-click action
        :return:
        """
        cloud_wrapper = HomeveeCloudWrapper()
        message_data = {'title': message_title, 'msg': message_body, 'clickaction': click_action}
        cloud_wrapper.send_push_notification(registration_ids, message_data)

    def send_notification_to_users(self, users: list, message_title: str, message_body: str,
                                   db: Database = None, click_action: str = None):
        """
        Sends a notification to the given users
        :param users: list of users
        :param message_title: the title of the message
        :param message_title: the title of the notification
        :param message_body: the body of the notification
        :param click_action: the on-click action
        :return:
        """
        registration_ids = self.get_user_tokens(users, db)
        self.send_notification(registration_ids, message_title, message_body, click_action)

    def send_notification_to_admin(self, message_title: str, message_body: str, db: Database,
                                   click_action: str = None):
        """
        Sends a notification to all admins
        :param message_title: the title of the message
        :param message_title: the title of the notification
        :param message_body: the body of the notification
        :param click_action: the on-click action
        :return:
        """
        admins = User.load_by_permission("admin", db)
        self.send_notification_to_users(admins, message_title, message_body, db, click_action)

    def get_user_tokens(self, users, db: Database = None):
        if db is None:
            db = Database()
        """
        Gets the users tokens
        :param users: the users
        :param db: the database connection
        :return: the list of tokens
        """
        tokens = []

        if users is None or (len(users) is 0):
            items = db.select_all("SELECT TOKEN FROM PUSH_NOTIFICATION_TOKENS", None)

            for item in items():
                tokens.append(item['TOKEN'])
        else:
            for user in users:
                item = db.select_one("SELECT TOKEN FROM PUSH_NOTIFICATION_TOKENS WHERE USERNAME = :user",
                            {'user': user})

                if (item['TOKEN'] is not None):
                    tokens.append(item['TOKEN'])

        return tokens

    def delete_user_tokens(self, username: str, db: Database = None):
        if db is None:
            db = Database()
        """
        Deletes the user token
        :param username: the user to delete the token from
        :param db: the database connection
        :return:
        """
        db.delete("DELETE FROM PUSH_NOTIFICATION_TOKENS WHERE USERNAME = :user", {'user': username})


from Homevee.Exception import AbstractFunctionCallException


class Condition():
    def __init__(self):
        return

    def is_true(self) -> bool:
        """
        Checks if the Condition is met
        :return: bool
        """
        raise AbstractFunctionCallException("Condition.is_true() is abstract")


#!/usr/bin/python
# -*- coding: utf-8 -*-
import time
from _thread import start_new_thread

from Homevee.Helper import Logger


class Cronjob():
    '''
        task_name => the name of the task
    '''

    def __init__(self, task_name):
        self.task_name = task_name

        start_new_thread(self.start_task, ())

    def start_task(self):
        """
        Start the task to do
        :return:
        """
        while (True):
            start_time = time.time()
            self.task_to_do()
            end_time = time.time()
            execution_time = end_time - start_time
            seconds_to_wait = self.get_seconds_to_wait(execution_time)
            Logger.log("Running Cronjob: " + self.task_name +" in "+str(int(seconds_to_wait))+" seconds")
            time.sleep(seconds_to_wait)

    def task_to_do(self, *args):
        """
        Executes the task to do
        :param args: additional arguments
        :return:
        """
        raise NotImplementedError("This Method has to be overridden")

    def get_seconds_to_wait(self, execution_time=None) -> int:
        """
        Compute the number of seconds the cronjob has to wait before starting again
        :param execution_time: the time since the last execution
        :return: the number of seconds to wait
        """
        raise NotImplementedError("This Method has to be overridden")


class IntervalCronjob(Cronjob):
    '''
        interval_seconds => time in seconds to wait
        task_name => the name of the task
        wait_full_interval => waits the given time AFTER the task if true,
                                    subtracts the tasks execution time if false
    '''
    def __init__(self, task_name, interval_seconds, wait_full_interval=False):
        super(IntervalCronjob, self).__init__(task_name)
        self.interval_seconds = interval_seconds
        self.wait_full_interval = wait_full_interval

    def get_seconds_to_wait(self, execution_time=None):
        seconds_to_wait = self.interval_seconds

        if (not self.wait_full_interval):
            seconds_to_wait = seconds_to_wait - execution_time

        return seconds_to_wait


class FixedTimeCronjob(Cronjob):
    def __init__(self, task_name):
        super(FixedTimeCronjob, self).__init__(task_name)

    def start_task(self):
        while (True):
            seconds_to_wait = self.get_seconds_to_wait()

            Logger.log("Running Cronjob: " + self.task_name +" in "+str(int(seconds_to_wait))+" seconds")

            time.sleep(seconds_to_wait)

            self.task_to_do()

    def get_seconds_to_wait(self, execution_time=None):
        raise NotImplementedError("This Method has to be overridden")


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee import Helper, Updater
from Homevee.DeviceAPI.set_modes import set_modes
from Homevee.Functions import ar_control, people_classifier
from Homevee.Helper import translations
from .DeviceAPI import dimmer, energy_data
from .DeviceAPI.blind_control import set_blinds, get_all_blinds, set_room_blinds
from .DeviceAPI.get_modes import *
from .DeviceAPI.heating import get_thermostats, control_room_heating, heating_control
from .DeviceAPI.rfid_control import get_rfid_tags, add_edit_rfid_tag, run_rfid_action, delete_rfid_tag
from .DeviceAPI.rgb_control import rgb_control
from .DeviceAPI.wake_on_lan import *
# from Functions import people_classifier, ar_control
# from Functions.people_classifier import *
from .Functions import room_data, weather, events, chat, shopping_list, media_center, gps_data, nutrition_data
from .Functions.calendar import *
from .Functions.condition_actions.actions import run_scene
from .Functions.graph_data import *
from .Functions.home_budget import get_home_budget_data, add_edit_home_budget_data, delete_home_budget_data, \
    get_home_budget_data_day_items, get_home_budget_data_graph
from .Functions.sensor_data import *
from .Functions.system_info import *
from .Functions.tv_data import *
from .Helper.helper_functions import load_remote_data, save_remote_data, set_remote_control_enabled, \
    connect_remote_id_with_account
from .Manager import dashboard, mqtt_connector, places
from .Manager.api_key import *
from .Manager.automation import *
from .Manager.custom_voice_commands import get_voice_commands, add_edit_voice_command, delete_voice_command
from .Manager.device import *
from .Manager.gateway import *
from .Manager.heating_scheme import *
from .Manager.persons import get_persons, add_edit_person
from .Manager.room import *
from .Manager.scenes import add_edit_scene, delete_scene, get_all_scenes
from .Manager.smart_speaker import get_smart_speakers
from .Manager.user import *
from .VoiceAssistant import *
from .VoiceAssistant.VoiceReplaceManager import *


class API:
    def __init__(self):
        return

    def process_data(self, data: dict, db: Database) -> dict:
        """
        Process the request
        :param data: the data dict
        :param db: the database connection
        :return: the response data dict
        """
        try:
            username = data['username']
            password = data['password']

            user = User.load_username_from_db(username, db)
            verified = user.verify(password)

            if 'language' in data and data['language'] is not None:
                language = data['language']
            else:
                language = translations.LANGUAGE

            if not verified:
                return json.dumps({'result': 'wrongdata'})

            try:
                return self.handle_request(user, data, db)
            except Exception as e:
                if(Logger.IS_DEBUG):
                    traceback.print_exc()
                return None
        except Exception as e:
            if(Logger.IS_DEBUG):
                    traceback.print_exc()
            return Status(type=STATUS_ERROR).get_dict()


    def handle_request(self, user, request, db: Database = None):
        if db is None:
            db = Database()
        response = None

        #print(user, request)

        action = request['action']
        username = request['username']

        if 'language' in request:
            language = request['language']
        else:
            language = translations.LANGUAGE

        if ('user_last_location' in request and request['user_last_location'] is not None):
            user_last_location = json.loads(request['user_last_location'])
        else:
            user_last_location = None

        # Roomdata
        if action == "login":
            rooms = room_data.get_rooms(user, db)

            remote_id = db.get_server_data("REMOTE_ID")

            response = {'remote_id': remote_id, 'rooms': rooms['rooms']}

        # Roomdata
        elif action == "getrooms":
            response = room_data.get_rooms(user, db)
        elif action == "getroomdata":
            response = room_data.get_room_data(user, request['room'], db)

        # Remote-ID
        elif action == "setremotedata":
            response = save_remote_data(user, request['remote_id'], request['linked_account'], db)
        elif action == "getremotedata":
            response = load_remote_data(user, db)
        elif action == "setremotecontrolenabled":
            response = set_remote_control_enabled(user, request['enabled'], db)
        elif action == "connectremoteidwithaccount":
            response = connect_remote_id_with_account(user, request['accountname'], request['accountsecret'], db)

        # Kalender
        elif action == "getcalendaritemdates":
            response = get_calendar_item_dates(user, request['year'], db)
        elif action == "getcalendardayitems":
            response = get_calendar_day_items(user, request['date'], db)
        elif action == "addeditcalendarentry":
            response = add_edit_entry(user, request['id'], request['name'], request['date'], request['start'],
                                      request['end'], request['note'], request['address'], db)
        elif action == "deletecalendarentry":
            response = delete_entry(user, request['id'], db)

        # Personen
        elif action == "getpersons":
            response = get_persons(db)
        elif action == "addeditperson":
            response = add_edit_person(user, request['id'], request['name'], request['nickname'], request['address'],
                                       request['latitude'], request['longitude'], request['phonenumber'],
                                       request['birthdate'], db)

        # RGB Lampen
        elif action == "setrgb":
            response = rgb_control(user, request['type'], request['id'], request['mode'], request['brightness'],
                                   request['color'], db)

        # RFID
        elif action == "getrfidtags":
            response = get_rfid_tags(user, db)
        elif action == "addeditrfidtag":
            response = add_edit_rfid_tag(user, request['name'], request['uuid'], request['type'], request['id'], db)
        elif action == "deleterfidtag":
            response = delete_rfid_tag(user, request['uuid'], db)
        elif action == "runrfidaction":
            response = run_rfid_action(user, request['uuid'], db)

        # Events
        elif action == "getevents":
            type = None
            if "type" in request:
                type = request['type']
            response = events.get_events(user, type, request['limit'], request['offset'], db)
        elif action == "geteventtypes":
            response = events.get_event_types(db)
        elif action == "addevent":
            response = events.add_event(request['type'], request['text'], db)
        elif action == "getunseenevents":
            response = events.get_unseen_events(user, db)

        # Permissions
        elif action == "getpermissions":
            response = Helper.permissions.get_permissions(request['user'], db)
        elif action == "setpermissions":
            response = Helper.permissions.set_permissions(user, request['user'], request['permissions'], db)

        # MQTT Anlernen => generate_device_data()
        elif action == "generatedevicedata":
            response = mqtt_connector.generate_device_data(user, request['location'], db)
        elif action == "savemqttdevice":
            response = mqtt_connector.save_mqtt_device(user, request['type'], request['location'], request['id'],
                                                       request['devicedata'], db)

        # Jalousie-Steuerung
        elif action == "setblinds":
            response = set_blinds(user, request['type'], request['id'], request['value'], db)
        elif action == "setroomblinds":
            response = set_room_blinds(user, request['location'], request['value'], db)
        elif action == "getallblinds":
            response = get_all_blinds(user, db)

        # Weather
        elif action == "getweather":
            response = weather.get_weather(int(request['daycount']), db)
        elif action == "getweathercitylist":
            response = weather.get_weather_city_list(db)
        elif action == "setweathercityid":
            response = weather.set_weather_city_id(user, request['id'])

        # Get graph data
        elif action == "getgraphdata":
            response = get_graph_data(user, request['room'], request['type'], request['id'], request['von'], request['bis'],
                                      db)

        # Get modes, set modes
        elif action == "getmodes":
            response = get_modes(user, request['room'], request['type'], request['device'], db)
        elif action == "setmodes":
            response = set_modes(user, request['type'], request['device'], request['zustand'], db)

        # Dimmer
        elif action == "setdimmer":
            response = dimmer.set_dimmer(user, request['type'], request['id'], request['value'], db)

        # Wake on Lan
        elif action == "wakeonlan":
            response = wake_on_lan(user, request['id'], db)
        elif action == "startxboxone":
            response = wake_xbox_on_lan(user, request['id'], db)

        # Systeminfo
        elif action == "getsysteminfo":
            response = get_system_info()

        # Chat
        elif action == "getchatmessages":
            response = chat.get_chat_messages(user, request['time'], request['limit'], db)
        elif action == "getchatimage":
            response = chat.get_chat_image(user, request['imageid'], db)
        elif action == "sendchatmessage":
            response = chat.send_chat_message(user, request['data'], db)

        # Dashboard
        elif action == "getuserdashboarditems":
            response = dashboard.get_user_dashboard_items(user, db)
        elif action == "getuserfavouritedevices":
            response = dashboard.get_user_favourite_devices(user, db)
        elif action == "edituserdashboard":
            response = dashboard.edit_user_dashboard(user, request['dashboarddata'], db)
        elif action == "getuserdashboard":
            response = dashboard.get_user_dashboard(user, db)

        # Energie-Daten
        elif action == "getenergydata":
            response = energy_data.get_energy_data(user, request['room'], request['devicetype'], request['deviceid'],
                                                   request['von'], request['bis'], db)
        elif action == "getdeviceenergydata":
            response = energy_data.get_device_energy_data(user, request['room'], request['devicetype'], request['deviceid'],
                                                          request['von'], request['bis'], db)
        elif action == "getenergycourse":
            response = energy_data.get_energy_course(user, request['room'], request['von'], request['bis'], db)
        elif action == "setpowercost":
            response = energy_data.set_power_cost(user, request['cost'], db)
        elif action == "getpowercost":
            response = energy_data.get_power_cost(user, db)

        # Sensor-Daten
        elif action == "getsensordata":
            response = get_sensor_data(user, request['room'], db)
        # User-Manager
        elif action == "getusers":
            response = get_users(user, db)
        elif action == "deleteuser":
            response = delete_user(user, request['name'], db)
        elif action == "addedituser":
            response = add_edit_user(user, request['name'], request['psw'], request['ip'],
                                     request['permissions'], db)

        # Gateway-Manager
        elif action == "getgateways":
            response = get_gateways(user, db)
        elif action == "addeditgateway":
            response = add_edit_gateway(user, request['type'], request['usr'], request['psw'], request['changepw'],
                                        request['ip'], request['port'], request['gateway_type'], db)
        elif action == "deletegateway":
            response = delete_gateway(user, request['type'], db)
        elif action == "getgatewaydevices":
            response = get_gateway_devices(user, request['type'], db)
        elif action == "connectgateway":
            response = connect_gateway(user, request['type'], request['ip'], db)

        # TV-Programm
        elif action == "settvchannels":
            response = set_tv_channels(user, request['channels'], db)
        elif action == "gettvplan":
            response = get_tv_plan(user, request['time'], db)
        elif action == "getalltvchannels":
            response = get_all_tv_channels(user, db)

        # Einkaufsliste
        elif action == "getshoppinglist":
            response = shopping_list.get_shopping_list(user, db)
        elif action == "addeditshoppinglistitem":
            response = shopping_list.add_edit_shopping_list_item(user, request['id'],
                                                                 request['amount'], request['name'], db)
        elif action == "deleteshoppinglistitem":
            response = shopping_list.delete_shopping_list_item(user, request['id'], db)

        # Sprachassistent
        elif action == "voicecommand":
            response = VoiceAssistant().do_voice_command(user, request['text'], user_last_location, None, db, language)
            # response = do_voice_command(user, request['text'], user_last_location, None, db, language)
        elif action == "getvoicereplaceitems":
            response = get_voice_replace_items(user, db)
        elif action == "addeditvoicereplaceitem":
            response = add_edit_voice_replace_item(user, request['replacewith'], request['itemstoreplace'], db)
        elif action == "deletevoicereplaceitem":
            response = delete_voice_replace_item(user, request['replacewith'], db)

        # Smart Speakers
        elif action == "getsmartspeakers":
            response = get_smart_speakers(user, db)

        # Heizpläne
        elif action == "addeditheatingschemeitem":
            response = add_edit_heating_scheme_item(user, request['id'], request['time'], request['value'],
                                                    request['active'], request['days'], request['data'], db)
        elif action == "deleteheatingschemeitem":
            response = delete_heating_scheme_item(user, request['id'], db)
        elif action == "getheatingschemeitems":
            response = get_heating_scheme_items(user, request['day'], request['rooms'], db)
        elif action == "setheatingschemeitemactive":
            response = set_heating_scheme_item_active(user, request['id'], request['active'], db)
        elif action == "getheatingschemeitemdata":
            response = get_heating_scheme_item_data(user, request['id'], db)
        elif action == "setheatingschemeactive":
            response = set_heating_scheme_active(user, request['active'], db)
        elif action == "isheatingschemeactive":
            response = is_heating_scheme_active(user, db)

        # API-Keys
        elif action == "getapikeydata":
            response = get_all_api_key_data(user, db)
        elif action == "setapikey":
            response = set_api_key(user, request['service'], request['apikey'], db)

        # Media Center
        elif action == "getmediacenters":
            response = media_center.get_media_centers(user, db)
        elif action == "mediacentercontrol":
            response = media_center.media_remote_action(request['type'], request['id'], request['remoteaction'], db)
        elif action == "mediacentersendtext":
            response = media_center.media_center_send_text(request['type'], request['id'], request['text'], db)
        elif action == "mediacentermusic":
            response = media_center.get_media_center_music(request['type'], request['id'],
                                                           request['limit'], request['offset'], db)
        elif action == "mediacenterartists":
            response = media_center.get_media_center_artists(request['type'], request['id'],
                                                             request['limit'], request['offset'], db)
        elif action == "mediacenteralbums":
            response = media_center.get_media_center_albums(request['type'], request['id'],
                                                            request['limit'], request['offset'], db)
        elif action == "mediacentermusicgenres":
            response = media_center.get_media_center_music_genres(request['type'], request['id'],
                                                                  request['limit'], request['offset'], db)
        elif action == "mediacentertvshows":
            response = media_center.get_media_center_shows(request['type'], request['id'],
                                                           request['limit'], request['offset'], db)
        elif action == "mediacentertvshowseasons":
            response = media_center.get_media_center_show_seasons(request['type'], request['id'], request['limit'],
                                                                  request['offset'], request['showid'], db)
        elif action == "mediacentertvshowepisodes":
            response = media_center.get_media_center_show_episodes(request['type'], request['id'], request['limit'],
                                                                   request['offset'], request['showid'],
                                                                   request['seasonid'], db)
        elif action == "mediacentermovies":
            response = media_center.get_media_center_movies(request['type'], request['id'],
                                                            request['limit'], request['offset'], db)
        elif action == "mediacentermoviegenres":
            response = media_center.get_media_center_movie_genres(request['type'], request['id'],
                                                                  request['limit'], request['offset'], db)
        elif action == "mediacenterplaying":
            response = media_center.get_media_menter_playing(request['type'], request['id'], db)

        # Firebase-Token aktualisieren
        elif action == "updatefcmtoken":
            response = set_user_fcm_token(user, request['token'], db)

        # Heizung
        elif action == "heatingcontrol":
            response = heating_control(user, request['type'], request['id'], request['value'], db)
        elif action == "controlroomheating":
            response = control_room_heating(user, request['location'], request['value'], db)
        elif action == "getthermostats":
            response = get_thermostats(user, request['room'], db)

        # Räume
        elif action == "addeditroom":
            response = add_edit_room(user, request['roomname'], request['location'], request['icon'], db)
        elif action == "moveitemsanddeleteoldroom":
            response = move_items_and_delete_old_room(user, request['oldroom'], request['newroom'], db)
        elif action == "deleteroomwithitems":
            response = delete_room_with_items(user, request['location'], db)
        elif action == "deleteroom":
            response = delete_room(user, request['location'], db)

        # Geräte-Manager
        elif action == "addeditdevice":
            response = add_edit_device(user, request['id'], request['type'], request['data'], db)
        elif action == "deletedevice":
            response = delete_device(user, request['type'], request['id'], db)
        elif action == "getdevicedata":
            response = get_device_data(user, request['type'], request['id'], db)

        # AR-Control
        elif action == "arcontrol":
            response = ar_control.ar_control(user, request['imagedata'], db)
        elif action == "getarcontrolclasses":
            response = ar_control.get_ar_control_classes(db)
        elif action == "savearcontrolclass":
            response = ar_control.save_ar_control_class(user, request['id'], request['data'], request['classname'], db)
        elif action == "getarcontrolclassimages":
            response = ar_control.get_ar_control_class_images(request['class'], request['show'], request['offset'], db)
        elif action == "startarcontroltraining":
            response = ar_control.start_ar_training(user, db)
        elif action == "changearcontrolimageclass":
            response = ar_control.change_ar_image_class(request['ids'], request['newclass'], db)
        elif action == "getarcontrolimages":
            response = ar_control.get_ar_control_image(request['id'], db)
        elif action == "getarimageclassifierperformancesettings":
            response = ar_control.get_performance_settings(user, db)
        elif action == "setarimageclassifierperformancesettings":
            response = ar_control.set_performance_settings(user, request['data'], db)
        elif action == "uploadarcontrolimages":
            response = ar_control.upload_images(user, request['data'], request['class'], db)
        elif action == "deletearcontrolimages":
            response = ar_control.delete_ar_control_images(request['ids'], db)

        # Gesichtserkennung
        elif action == "getpeopleclasses":
            response = people_classifier.get_people_classes(db)
        elif action == "savepeopleclass":
            response = people_classifier.save_people_class(user, request['id'], request['data'], request['classname'], db)
        elif action == "startpeopletraining":
            response = people_classifier.start_people_training(user, db)
        elif action == "getpeopleimageclassifierperformancesettings":
            response = people_classifier.get_performance_settings(user, db)
        elif action == "setpeopleimageclassifierperformancesettings":
            response = people_classifier.set_performance_settings(user, request['data'], db)
        elif action == "changepeopleclassifierimageclass":
            response = people_classifier.change_people_image_class(request['ids'], request['newclass'], db)
        elif action == "getpeopleclassifierimage":
            response = people_classifier.get_people_image(request['id'], db)
        elif action == "getpeopleclassifierimages":
            response = people_classifier.get_people_images(request['class'], request['show'], request['offset'], db)
        elif action == "uploadpeopleimages":
            response = people_classifier.upload_images(user, request['data'], request['class'], db)
        elif action == "classifyperson":
            response = people_classifier.classify_person(request['imagedata'], db)
        elif action == "deletepeopleimages":
            response = people_classifier.delete_people_images(request['ids'], db)

        # Automation
        elif action == "getautomationrules":
            response = get_automations(user, request['location'], db)
        elif action == "addeditautomationrule":
            response = add_edit_automation_rule(user, request['location'], request['id'], request['name'],
                                                request['triggerdata'], request['conditiondata'], request['actiondata'],
                                                request['isactive'], db)
        elif action == "deleteautomationrule":
            response = delete_automation_rule(user, request['id'], db)
        # Eigene Sprachbefehle
        elif action == "getvoicecommands":
            response = get_voice_commands(db)
        elif action == "addeditvoicecommand":
            response = add_edit_voice_command(user, request['id'], request['name'], request['command_data'],
                                              request['response_data'], request['action_data'], db)
        elif action == "deletevoicecommand":
            response = delete_voice_command(user, request['id'], db)
        # Szenen
        elif action == "getallscenes":
            response = get_all_scenes(user, db)
        elif action == "getscenes":
            response = get_scenes(user, request['location'], db)
        elif action == "addeditscene":
            response = add_edit_scene(user, request['id'], request['name'], request['location'], request['action_data'], db)
        elif action == "deletescene":
            response = delete_scene(user, request['id'], db)
        elif action == "runscene":
            response = run_scene(user, request['id'], db)
        # Haushaltskasse
        elif action == "gethomebudgetdata":
            response = get_home_budget_data(user, request['startdate'], db)
        elif action == "gethomebudgetdatadayitems":
            response = get_home_budget_data_day_items(user, request['date'], db)
        elif action == "gethomebudgetdatagraph":
            response = get_home_budget_data_graph(user, request['startdate'], request['enddate'], db)
        elif action == "addedithomebudgetdata":
            response = add_edit_home_budget_data(user, request['id'], request['date'], request['info'], request['amount'],
                                                 db)
        elif action == "deletehomebudgetdata":
            response = delete_home_budget_data(user, request['id'], db)
        # GPS data
        elif action == "updategps":
            response = gps_data.update_gps(user, request['time'], request['lat'], request['lng'], db)
        elif action == "getgpslocations":
            response = gps_data.get_gps_locations(user, db)
        # Places
        elif action == "addeditplace":
            response = places.add_edit_place(user, request['id'], request['name'], request['address'], request['latitude'],
                                             request['longitude'], db)
        elif action == "getmyplaces":
            response = places.get_my_places(user, db)
        elif action == "deleteplace":
            response = places.delete_place(user, request['id'], db)
        # Ernährungsmanager
        elif action == "submitfood":
            response = nutrition_data.submit_food(user, request['name'], request['calories'], request['portionsize'],
                                                  request['portionunit'], request['protein'], request['fat'],
                                                  request['saturated'], request['unsaturated'], request['carbs'],
                                                  request['sugar'], request['ean'], db)
        elif action == "getusernutritionoverview":
            response = nutrition_data.get_user_nutrition_overview(user, db)
        elif action == "getuserdaynutritionitems":
            response = nutrition_data.get_user_day_nutrition_items(user, request['date'], db)
        elif action == "addedituserdaynutritionitem":
            response = nutrition_data.add_edit_user_day_nutrition_item(user, request['id'], request['date'],
                                                                       request['daytime'], request['name'],
                                                                       request['eatenportionsize'], request['portionsize'],
                                                                       request['portionunit'],
                                                                       request['calories'], request['fat'],
                                                                       request['saturated'], request['unsaturated'],
                                                                       request['carbs'], request['sugar'],
                                                                       request['protein'], db)
        elif action == "loadnutritionmanagersettings":
            response = nutrition_data.load_user_settings(user, db)
        elif action == "deletefooditem":
            response = nutrition_data.delete_food_item(user, request['id'], db)
        elif action == "savenutritionmanagersettings":
            response = nutrition_data.save_user_settings(user, request['height'], request['weight'], request['birthdate'],
                                                         request['mode'], request['activity'], db)
        elif action == "movenutritionitem":
            response = nutrition_data.move_item(user, request['id'], request['date'], request['daytime'],
                                                request['deleteold'], db)
        # Updates
        elif action == "checkforupdates":
            response = Updater.check_for_updates()
        elif action == "updatesystem":
            response = Updater.do_homevee_update(user, db)
        else:
            return {'result': 'nosuchaction'}

        # Ausgeben
        return json.dumps(response)

        '''
        //Kalender
            case "getcalendaritems":
                $result = getCalendarItems($_POST['cal_user'], $db);
                break;
            case "getcalendaritemsforday":
                $result = getCalendarItemsForDay($_POST['USERNAME'], $_POST['date'], $db);
                break;
            case "getcalendardataformonth":
                $result = getCalendarDataForMonth($_POST['USERNAME'], $_POST['month'], $db);
                break;
            case "getcalendariteminfo":
                $result = getCalendarItemInfo($_POST['id'], $db);
                break;
            case "addeditcalendaritem":
                $result = addEditCalendarItem($_POST['id'], $_POST['name'],
                $_POST['start'], $_POST['end'], $_POST['note'], $_POST['repeat'], $_POST['participants'], $db);
                break;
            case "deletecalendaritem":
                $result = deleteCalendarItem($_POST['id'], $db);
                break;
        '''

        '''
            case "runscene":
                $result = runScene($_POST['room'], $_POST['name'], $db);
                break;
            case "createscene":
                $result = createScene($_POST['devices'], $_POST['rooms'], $_POST['types'], $_POST['values'],
                    $_POST['conditions'], $_POST['room'], $_POST['name'], $db);
                break;
            case "getscenes":
                $result = getScenes($_POST['room'], $db);
                break;
    
            //Noch nicht im Tutorial
            case "shouldshowmorninginfo":
                //$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
                break;
            case "getmorninginfo":
                //$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
                break;
            case "getmorninginfosettings":
                //$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
                break;
            case "setmorninginfosettings":
                //$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
                break;
            case "rfidvalidation":
                $result = rfidValidation($_POST['uid'], $db);
                break;
            case "rfidtrigger":
                $result = rfidTrigger($_POST['uid'], $_POST['triggerid'], $db);
                break;
            case "heatingcontrol":
                $result = heatingControl($_POST['room'], $_POST['type'], $_POST['id'], $_POST['value'], $db);
                break;
            case "getheatingdata":
                $result = getHeatingData($_POST['room'], $_POST['type'], $_POST['id'], $_POST['all_data'], $db);
                break;
            case "controldiydevice":
                $result = diyDeviceControl($_POST['device'], $_POST['room'], $_POST['diyaction'], $_POST['value'], $db);
                break;
            case "getdiyinfo":
                $result = diyDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
                break;
            case "getreeddata":
                $result = getReedData($_POST['room'], $_POST['type'], $_POST['id'], $db);
                break;
            case "addedittargetid":
                $result = addEditTargetID($_POST['user'], $_POST['deviceid'], $_POST['targetid'], $db);
                break;
            case "getpowermeterdata":
                $result = getSensorData($_POST['room'], $_POST['type'], $_POST['id'], $_POST['hide_einheit'], $db);
                break;
            case "getdimmer":
                $result = getDimmer($_POST['room'], $_POST['type'], $_POST['id'], $db);
                break;
            case "setdimmer":
                $result = setDimmer($_POST['room'], $_POST['type'], $_POST['id'], $_POST['value'], $db);
                break;
    
            //Stromverbrauch
            case "getenergydata":
                $result = getEnergyData($_POST['room'], $_POST['devicetype'], $_POST['deviceid'], $_POST['von'], $_POST['bis'], $db);
                break;
            case "getdeviceenergydata":
                $result = getDeviceEnergyData($_POST['type'], $_POST['device'], $_POST['von'], $_POST['bis'], $db);
                break;
            case "setpowercost":
                $result = setPowerCost($_POST['cost'], $db);
                break;
            case "getpowercost":
                $result = getPowerCost($db);
                break;
    
            //Überwachungskamera
            case "getsurveillancefootage":
                $result = getSurveillanceFootage($_POST['id'], $_POST['filterdata'], $_POST['offset'], $_POST['limit'], $db);
                break;
            case "getsurveillancethumbnail":
                $result = getSurveillanceThumbnail($_POST['id'], $db);
                break;
            case "getsurveillancefootagevideo":
                $result = getSurveillanceFootageVideo($_POST['id'], $db);
                break;
            case "getipcameradata":
                $result = getIpCameraData($_POST['room'], $_POST['id'], $db);
                break;
    
            //Türspion
            case "addpeepholeimages":
                $result = addPeepholeImages($_POST['imagedata'], $db);
                break;
            case "getpeepholeclassimages":
                $result = getPeepholeClassImages($_POST['class'], $_POST['show'], $_POST['offset'], $db);
                break;
            case "changepeepholeimageclass":
                $result = changePeepholeClass($_POST['ids'], $_POST['newclass'], $db);
                break;
            case "getpeepholeimage":
                $result = getPeepholeImage($_POST['id'], $db);
                break;
            case "addeditpeepholeclass":
                $result = addEditPeepHoleClass($_POST['id'], $_POST['name'], $db);
                break;
            case "deletepeepholeclass":
                $result = deletePeepHoleClass($_POST['id'], $db);
                break;
    
            //Bild-Klassifizierer Einstellungen
            case "getimageclassifierperformancesettings":
                $result = getImageClassifierPerformanceSettings($db);
                break;
            case "setimageclassifierperformancesettings":
                $result = setImageClassifierPerformanceSettings($_POST['data'], $db);
                break;
    
            //Manager
            case "getdevicedata":
                $result = getDeviceData($_POST['type'], $_POST['id'], $db);
                break;
            case "deletedevice":
                $result = deleteDevice($_POST['type'], $_POST['id'], $db);
                break;
            case "addeditdevice":
                $result = addEditDevice($_POST['id'], $_POST['type'], $_POST['data'], $db);
                break;
    
            //Kalender
            case "getcalendaritems":
                $result = getCalendarItems($_POST['cal_user'], $db);
                break;
            case "getcalendaritemsforday":
                $result = getCalendarItemsForDay($_POST['USERNAME'], $_POST['date'], $db);
                break;
            case "getcalendardataformonth":
                $result = getCalendarDataForMonth($_POST['USERNAME'], $_POST['month'], $db);
                break;
            case "getcalendariteminfo":
                $result = getCalendarItemInfo($_POST['id'], $db);
                break;
            case "addeditcalendaritem":
                $result = addEditCalendarItem($_POST['id'], $_POST['name'],
                $_POST['start'], $_POST['end'], $_POST['note'], $_POST['repeat'], $_POST['participants'], $db);
                break;
            case "deletecalendaritem":
                $result = deleteCalendarItem($_POST['id'], $db);
                break;
    
            //MediaCenter
            case "getmediacenters":
                $result = getMediaCenters($db);
                break;
            case "mediacentercontrol":
                $result = remoteAction($_POST['type'], $_POST['id'], $_POST['remoteaction'], $db);
                break;
            case "mediacentersendtext":
                $result = mediaCenterSendText($_POST['type'], $_POST['id'], $_POST['text'], $db);
                break;
            case "mediacentermusic":
                $result = getMediaCenterMusic($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
                break;
            case "mediacenterartists":
                $result = getMediaCenterArtists($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
                break;
            case "mediacenteralbums":
                $result = getMediaCenterAlbums($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
                break;
            case "mediacentermusicgenres":
                $result = getMediaCenterMusicGenres($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
                break;
            case "mediacentertvshows":
                $result = getMediaCenterShows($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
                break;
            case "mediacentertvshowseasons":
                $result = getMediaCenterShowSeasons($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $_POST['showid'], $db);
                break;
            case "mediacentertvshowepisodes":
                $result = getMediaCenterShowEpisodes($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $_POST['showid'], $_POST['seasonid'], $db);
                break;
            case "mediacentermovies":
                $result = getMediaCenterMovies($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
                break;
            case "mediacentermoviegenres":
                $result = getMediaCenterMovieGenres($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
                break;
            case "mediacenterplaying":
                $result = getMediaCenterPlaying($_POST['type'], $_POST['id'], $db);
                break;
    
            //Update
            case "checkforupdates":
                $result = checkForUpdates($db);
                break;
            case "updatesystem":
                $result = updateSystem($db);
                break;
            '''


from Homevee.Exception import AbstractFunctionCallException
from Homevee.Item.Logic.Action import Action


class ControlDeviceAction(Action):
    def __init__(self):
        super(ControlDeviceAction, self).__init__()

    def run(self):
        raise AbstractFunctionCallException("ControlDeviceAction.run() is abstract")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from Homevee.Utils import Constants


def get_translations():
    translations = {
        'no_users_create_admin': "Es wurden noch keine Benutzer erstellt.\Du kannst jetzt einen Administrator erstellen.",
        'enter_password': 'Gib ein Passwort ein: ',
        'password_dont_match': 'Die Passwörter stimmen nicht überein',
        'enter_username': 'Gib einen Nuternamen ein: ',
        'your_remote_id_is': 'Deine Remote-ID lautet: ',
        'homevee_server_started': "Homevee-Server (Version: " + Constants.HOMEVEE_VERSION_NUMBER + ") wurde gestartet...",
        'update_available': "Es ist ein Update auf eine neue Version ({}) von Homevee verfügbar.",
        'no_cloud_connection': "Keine Verbindung zu Homevee-Cloud möglich..."
    }
    return translations


import json

from Homevee.Helper.HomeveeCloud.CloudAPI import CloudAPI
from Homevee.Helper.HomeveeCloud.Exception import APINotAuthenticatedException, APIErrorException
from Homevee.Helper.HomeveeCloud.Response import Response


class HomeveeCloudWrapper():
    def __init__(self, remote_id=None, access_token=None):
        self.cloud_api = CloudAPI(remote_id, access_token)

        self.remote_id = self.cloud_api.remote_id
        self.access_token = self.cloud_api.access_token

    def is_premium(self) -> bool:
        """
        Checks if the remote-id is premium
        :return:
        """
        response = self.cloud_api.do_get("/ispremium/"+self.remote_id, {})
        data = self.check_response(response)
        return data['is_premium']

    def send_push_notification(self, registration_ids: list, message_data: dict):
        """
        Sends a push notification to the given registration ids
        :param registration_ids: the recipients ids
        :param message_data: the data of the message
        :return:
        """
        response = self.cloud_api.do_put("/sendnotification",
                                         {'registration_ids': registration_ids,
                                          'message_data': message_data})
        data = self.check_response(response)
        return data['status']

    def set_ip(self, ip: str):
        """
        Sets the homevee-hubs local ip address
        :param ip: the ip address
        :return:
        """
        response = self.cloud_api.do_put("/setlocalip/"+self.remote_id, {'ip': ip})
        data = self.check_response(response)
        return data['status']

    def set_cert(self, cert: str):
        """
        Sets the homevee-hubs local certificates
        :param cert: the certificate
        :return:
        """
        response = self.cloud_api.do_put("/setlocalcert/"+self.remote_id, {'cert': cert})
        data = self.check_response(response)
        return data['status']

    def get_ip(self):
        """
        Get the local ip address of the hub
        :return: the ip
        """
        response = self.cloud_api.do_get("/getlocalip/"+self.remote_id, {})
        data = self.check_response(response)
        return data['local_ip']

    def get_cert(self):
        """
        Gets the local certificate of the hub
        :return: the certificate
        """
        response = self.cloud_api.do_get("/getlocalcert/"+self.remote_id, {})
        data = self.check_response(response)
        return data['local_cert']

    def check_response(self, response: Response) -> dict:
        """
        Checks the response and returns its data if valid
        :param response: the response to check
        :return: the dict with the responses data
        """
        if response is None:
            raise APIErrorException("API-Call not successful")

        if response.status_code == 401:
            raise APINotAuthenticatedException("Invalid credentials given")

        if response.status_code != 200:
            raise APIErrorException("API-Call not successful")

        return json.loads(response.response)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Device import Device
from Homevee.Item.Device.Thermostat.MaxThermostat import *
from Homevee.Item.Device.Thermostat.RademacherThermostat import *
from Homevee.Item.Device.Thermostat.ZWaveThermostat import *
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Utils.DeviceTypes import *


def heating_control(user, type, id, value, db, check_user=True):
    if type == MAX_CUBE:
        module = MaxThermostat
    elif type == ZWAVE_THERMOSTAT:
        module = ZWaveThermostat
    elif type == RADEMACHER_THERMOSTAT:
        module = RademacherThermostat
    else:
        return Status(type=STATUS_NO_SUCH_TYPE).get_dict()

    thermostat = Device.load_from_db(module, id)

    if check_user and not user.has_permission(thermostat.location):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    if(thermostat.set_temp(value)):
        return Status(type=STATUS_OK).get_dict()
    else:
        return Status(type=STATUS_ERROR).get_dict()

def control_room_heating(user, room, value, db: Database = None):
    if db is None:
        db = Database()
        devices = get_thermostats(user, room, "", "")['thermostats'][0]['thermostat_array']
        for device in devices:
            id = device['id']
            type = device['type']
            result = heating_control(user, type, id, value, db)
            if 'result' not in result or result['result'] != "ok":
                return Status(type=STATUS_ERROR).get_dict()
        return Status(type=STATUS_OK).get_dict()

def get_thermostat_info(user, room, type, id, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    module_map = {
        ZWAVE_THERMOSTAT: ZWaveThermostat,
        MAX_THERMOSTAT: MaxThermostat,
        RADEMACHER_THERMOSTAT: RademacherThermostat
    }

    if type not in module_map:
        return Status(type=STATUS_NO_SUCH_TYPE).get_dict()

    thermostat = Device.load_from_db(module_map[type], id)

    min, max = thermostat.get_min_max()
    return {'value': thermostat.temp, 'min': min, 'max': max}

def get_thermostats(user, room, db: Database = None):
    if db is None:
        db = Database()
    if isinstance(room, Room):
        rooms = [room]
    elif room == "all":
        rooms = Room.load_all(db)
    else:
        rooms = Room.load_all_ids_from_db([room])

    thermostats = []

    for room in rooms:
        if not user.has_permission(room.id):
            continue

        thermostat_array = []

        modules = [ZWaveThermostat, MaxThermostat, RademacherThermostat]

        for module in modules:
            thermostat_items = Device.get_all(module, room)
            for thermostat in thermostat_items:
                min, max = thermostat.get_min_max()
                thermostat_data = {'name': thermostat.name, 'id': thermostat.id, 'type': thermostat.get_device_type(),
                                   'icon': thermostat.icon, 'data': {'value': thermostat.temp, 'min': min, 'max': max}}
                thermostat_array.append(thermostat_data)

        room_thermostats = {'name': room.name, 'location': room.id, 'icon': room.icon,
                            'thermostat_array': thermostat_array}
        thermostats.append(room_thermostats)

    return {'thermostats': thermostats}



from Homevee.VoiceAssistant import VoiceAssistant

def do_voice_command(username, text, user_last_location, location, db, language):
    return VoiceAssistant().do_voice_command(username, text, user_last_location, location, db, language)


from Homevee import Updater
from Homevee.Cronjob import IntervalCronjob

class CheckUpdateCronjob(IntervalCronjob):
    def __init__(self):
        super(CheckUpdateCronjob, self).__init__(task_name="CheckUpdateCronjob", interval_seconds=60*60)

    def task_to_do(self, *args):
        self.check_for_updates()

    def check_for_updates(self):
        new_version = Updater.get_homevee_update_version()

        #notify user

        #check if auto update is enabled and update accordingly


import json
from time import sleep

from iothub_client.iothub_client import IoTHubTransportProvider, IoTHubModuleClient, DeviceMethodReturnValue

from Homevee import API
from Homevee.Utils.Database_NEW import Database

METHOD_PROCESS_DATA = "ProcessData"

class CloudConnection():
    def __init__(self):
        self.protocol = IoTHubTransportProvider.MQTT

        self.msg_timeout = 1000
        self.timeout = 1000

        db = Database()

        cloud_host_name = "Homevee-Cloud.azure-devices.net"
        device_id = db.get_server_data("REMOTE_ID")
        shared_access_key = db.get_server_data("CLOUD_ACCESS_KEY")

        self.connection_string = "HostName="+cloud_host_name+";DeviceId="+device_id+\
                                 ";SharedAccessKey="+shared_access_key

        self.client = self.init_client()

        return

    # This function will be called every time a method request is received
    def method_callback(self, method_name, payload, user_context):
        retval = DeviceMethodReturnValue()

        if method_name == METHOD_PROCESS_DATA:
            try:
                data = json.loads(payload)

                msg = API().process_data(data, Database())

                print("CLOUD_COMMAND => "+payload)
                print("RESPONSE => "+msg)
                print("##############")

                retval.response = msg
                retval.status = 200
            except:
                retval.status = 500
                retval.response = "{ \"Response\": \"Could not call method %s\" }" % METHOD_PROCESS_DATA
        else:
            retval.status = 404
            retval.response = "{ \"Response\": \"Method %s not found\" }" % method_name

        return retval

    def init_client(self):
        print(self.connection_string)
        client = IoTHubModuleClient(self.connection_string, self.protocol)
        #client.create_from_environment(IoTHubTransportProvider.MQTT)
        #client.set_module_method_callback(self.method_callback, 0)
        client.set_module_method_callback(self.method_callback, 0)
        return client

    @staticmethod
    def cloud_connection_loop():
        cloud_connection = CloudConnection()
        while(True):
            sleep(15)

if __name__ == "__main__":
    CloudConnection.cloud_connection_loop()


#!/usr/bin/python
# -*- coding: utf-8 -*-

WHITE = "000000"

RED = "FF0000"
GREEN = "00FF00"
BLUE = "0000FF"

LIGHT_RED = "FF7F7F"
LIGHT_GREEN = "9BFF70"
LIGHT_BLUE = "00FFFF"

COLOR_NAMES = {
    'grün': GREEN,
    'rot': RED,
    'blau': BLUE,
    'hellgrün': LIGHT_GREEN,
    'hellblau': LIGHT_BLUE,
    'hellrot': LIGHT_RED,
    'weiß': WHITE
}

COLORS = list(COLOR_NAMES.keys())


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Functions.chefkoch_api import search_recipes
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceGetRecipesModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['zeig', 'mir', 'rezepte', ['über', 'für', 'zu', 'von']]
        ]

    def get_label(self):
        return "recipes"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.voice_recipes(username, text, context, db)

    def voice_recipes(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        words = text.split(" ")

        keyword = None
        found = False
        for word in words:
            if found:
                if keyword is None:
                    keyword = word
                else:
                    keyword = keyword + " " + word

            if word in ['über', 'für', 'zu', 'von']:
                found = True

        recipes = search_recipes(keyword, 5)

        recipe_data = []

        for item in recipes:
            recipe = item['recipe']

            recipe_data.append({'name':recipe['title']})

        text = None
        for item in recipe_data:
            if text is None:
                text = item['name']
            else:
                text = text + ", " + item['name']

        return {'msg_speech': 'Rezepte: '+text, 'msg_text': 'Rezepte: '+text}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules import VoiceModule

class VoiceGetSummaryModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        #todo declare pattern for summary
        return []

    def get_label(self):
        return "getsummary"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        pass

    def get_voice_summary(username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return {'msg_speech': 'Get Summary', 'msg_text': 'Get Summary'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI.rgb_control import rgb_control
from Homevee.Utils.Colors import COLORS, COLOR_NAMES
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *
from Homevee.VoiceAssistant.Modules.DeviceControlModule import VoiceDeviceControlModule


class VoiceRgbDeviceControlModule(VoiceDeviceControlModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            [['mach', 'stell', 'tu', 'dreh'], COLORS]
        ]

    def get_label(self):
        return "rgb"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.voice_rgb_control(username, text, context, db)

    def voice_rgb_control(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        room = self.find_room(text, db)

        if room is not None:
            room_key = room['LOCATION']
        else:
            room_key = None

        device_types = [PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]

        devices = self.find_devices(text, device_types, room_key, db)

        color = self.find_color(text)
        if color != False:
            color_hex = COLOR_NAMES[color]
        else:
            return "Du musst mir eine Farbe sagen."

        for device in devices:
            rgb_control(username, device['type'], device['id'], True, None, color_hex, db)

            return {'msg_speech': 'Ok.', 'msg_text': 'Ok.'}

    def find_color(self, text):
        for word in text.split():
            for color in COLORS:
                if word == color:
                    return color
        return False


from Homevee.Exception import InvalidParametersException, NoSuchTypeException
from Homevee.Item import Item
from Homevee.Item.Device.Sensor.MQTTSensor import MQTTSensor
from Homevee.Item.Device.Sensor.ZWaveSensor import ZWaveSensor
from Homevee.Item.Device.Switch.Funksteckdose import Funksteckdose
from Homevee.Item.Device.Switch.URLSwitch import URLSwitch
from Homevee.Item.Device.Thermostat.MaxThermostat import MaxThermostat
from Homevee.Item.Device.Thermostat.RademacherThermostat import RademacherThermostat
from Homevee.Item.Device.Thermostat.ZWaveThermostat import ZWaveThermostat
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


class DeviceLoader():
    def __init__(self):
        pass

    def load_devices(self, device_data: dict) -> list:
        """
        Loads devices identified by the device data dict
        :param device_data: dict of {'device_type': device_type, 'id': id}
        :return: list of devices
        """
        items = []
        db_con = Database()
        try:
            for data_item in device_data:
                id = data_item['id']
                if data_item['device_type'] == FUNKSTECKDOSE:
                    module = Funksteckdose
                elif data_item['device_type'] == URL_SWITCH:
                    module = URLSwitch
                elif data_item['device_type'] == MAX_THERMOSTAT:
                    module = MaxThermostat
                elif data_item['device_type'] == RADEMACHER_THERMOSTAT:
                    module = RademacherThermostat
                elif data_item['device_type'] == ZWAVE_THERMOSTAT:
                    module = ZWaveThermostat
                elif data_item['device_type'] == ZWAVE_SENSOR:
                    module = ZWaveSensor
                elif data_item['device_type'] == MQTT_SENSOR:
                    module = MQTTSensor
                else:
                    raise NoSuchTypeException("unkown type: "+data_item['device_type'])
                item = Item.load_from_db(module, id, db_con)
                items.append(item)
            return items
        except:
            raise InvalidParametersException("invalid params for DeviceLoader.load_devices()")



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

import requests

from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Item.Gateway import Gateway, Z_WAVE_GATEWAY
from Homevee.Utils.Database_NEW import Database


def get_rooms():
    result = {}

    gateway = Item.load_from_db(Gateway, Z_WAVE_GATEWAY, Database())

    port = ""
    if (gateway.port != None and gateway.port != ""):
        port = ":" + str(gateway.port)

    url = "http://" + gateway.key1 + ':' + gateway.key2 + '@' + gateway.ip\
          + port + "/ZAutomation/api/v1/locations/"
    headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
    request = requests.get(url, headers=headers)
    data = request.content

    Logger.log(data)

    result = json.loads(data)

    return result


from Homevee.Item import Item


class SystemInfoItem(Item):
    def __init__(self, name, icon, value):
        super(SystemInfoItem, self).__init__()
        self.name = name
        self.icon = icon
        self.value = value


import traceback

from Homevee.DeviceAPI.zwave.utils import do_zwave_request
from Homevee.Exception import DatabaseSaveFailedException, InvalidParametersException
from Homevee.Helper import Logger
from Homevee.Item.Device.Thermostat import Thermostat
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import ZWAVE_THERMOSTAT


class ZWaveThermostat(Thermostat):
    def __init__(self, name, icon, location, id=None, temp=None):
        super(ZWaveThermostat, self).__init__(name, icon, location, id=id, temp=temp)

    def get_device_type(self):
        return ZWAVE_THERMOSTAT

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM ZWAVE_THERMOSTATS WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def get_min_max(self):
        return 4, 40

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            # insert
            db.insert("""INSERT OR IGNORE INTO ZWAVE_THERMOSTATS (THERMOSTAT_ID, NAME, ICON, VALUE, RAUM) VALUES 
                        (:id, :name, :icon, :last_temp, :room)""",
                        {'id': self.id, 'name': self.name, 'icon': self.icon,
                         'last_temp': self.temp, 'room': self.location})
            # update
            db.update("""UPDATE OR IGNORE ZWAVE_THERMOSTATS SET NAME = :name, ICON = :icon,
                        VALUE = :last_temp, RAUM = :room WHERE THERMOSTAT_ID = :id""",
                        {'name': self.name, 'icon': self.icon, 'last_temp': self.temp,
                        'room': self.location, 'id': self.id})

                # TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save max-thermostat to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'id': self.id,
            'temp': self.temp,
            'location': self.location
        }
        return dict

    def update_temp(self, temp, db: Database = None):
        if db is None:
            db = Database()
        try:
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + str(self.id)
                                      + "/command/exact?level=" + str(temp),db)
            if result is not None and result['code'] == 200:
                return True
            else:
                return False
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            return False

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return ZWaveThermostat.load_all_from_db('SELECT * FROM MAX_THERMOSTATS WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = ZWaveThermostat(result['NAME'], result['ICON'], result['RAUM'], result['THERMOSTAT_ID'],
                                 result['VALUE'])
            items.append(item)
        return items

    @staticmethod
    def load_all(db: Database = None):
        if db is None:
            db = Database()
        return ZWaveThermostat.load_all_from_db('SELECT * FROM ZWAVE_THERMOSTATS', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            temp = dict['temp']
            icon = dict['icon']

            item = ZWaveThermostat(name, icon, location, id, temp)

            return item

        except:
            raise InvalidParametersException("Invalid parameters for ZWaveThermostat.create_from_dict()")


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database


def add_trigger(user, location, id, db: Database = None):
    if db is None:
        db = Database()
    return

def get_triggers(user, room, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return {'result': 'nopermission'}

    triggers = []

    results = db.select_all("SELECT * FROM MQTT_TRIGGERS WHERE LOCATION = :location", {'location': room})

    for item in results:
        triggers.append({'name': item['NAME'], 'id': item['ID'], 'type': 'MQTT-Trigger', 'icon': item['ICON']})

    return {'triggers': triggers}